CREATE TABLE ESD_CENS_ANY 
(
  CEN_CODCEN VARCHAR2(20) NOT NULL 
, NUMANY NUMBER(4) NOT NULL 
, PRCPRTMIG NUMBER(5, 2) DEFAULT 0 NOT NULL 
, NUMVOTCEN NUMBER(9) DEFAULT 0 NOT NULL 
, CONSTRAINT PK_ESD_CENS_ANY PRIMARY KEY 
  (
    CEN_CODCEN 
  , NUMANY 
  )
  ENABLE 
);

CREATE TABLE ESD_CIUTADA 
(
  NUMPRTVOT NUMBER(9) NOT NULL 
, CTD_CODCTD VARCHAR2(9) NOT NULL 
, CONSTRAINT RP_ECT PRIMARY KEY 
  (
    CTD_CODCTD 
  )
  ENABLE 
);

CREATE TABLE ESD_ANY 
(
  NUMANY NUMBER(4) NOT NULL 
, VOT_CODVOT VARCHAR2(10) 
, EST_CODEST VARCHAR2(2) 
, REG_CODREG VARCHAR2(9) 
, LOC_CODLOC VARCHAR2(9) 
, PRCCTDNOV NUMBER(5, 2) DEFAULT 0 NOT NULL 
, NUMMAXVOT NUMBER(9) 
, CONSTRAINT PK_ESD_ANY PRIMARY KEY 
  (
    NUMANY 
  )
  ENABLE 
);

CREATE TABLE ESD_LOCALITAT_ANY 
(
  NUMANY NUMBER(4, 0) NOT NULL 
, EST_CODEST VARCHAR2(2) NOT NULL 
, REG_CODREG VARCHAR2(9) NOT NULL 
, LOC_CODLOC VARCHAR2(9) NOT NULL 
, NUMVOT NUMBER(9) NOT NULL 
, CONSTRAINT RP_ELA PRIMARY KEY 
  (
    EST_CODEST 
  , REG_CODREG 
  , LOC_CODLOC 
  , NUMANY 
  )
  ENABLE 
);

CREATE TABLE ESD_VOTACIO 
(
  VOT_CODVOT_MAXPRT VARCHAR2(10) 
, VOT_CODVOT_MAXDIF VARCHAR2(10) 
, VOT_CODVOT_MINDIF VARCHAR2(10) 
, NUMMAXPRTCTD NUMBER(9) DEFAULT 0 NOT NULL 
, EST_CODEST_MAXPRTCTD VARCHAR2(2) 
);

CREATE TABLE ESD_ESTAT_ANY 
(
  EST_CODEST VARCHAR2(2) NOT NULL 
, NUMANY NUMBER(4) NOT NULL 
, NUMVOTFIN NUMBER(9) DEFAULT 0 NOT NULL 
, CONSTRAINT PK_ESD_ESTAT_ANY PRIMARY KEY 
  (
    EST_CODEST 
  , NUMANY 
  )
  ENABLE 
);

ALTER TABLE ESD_CENS_ANY
ADD CONSTRAINT ESD_CENS_ANY_CENS_FK FOREIGN KEY
(
  CEN_CODCEN 
)
REFERENCES CENS
(
  CODCEN 
)
ENABLE;

ALTER TABLE ESD_CIUTADA
ADD CONSTRAINT RF_ECT_CTD FOREIGN KEY
(
  CTD_CODCTD 
)
REFERENCES CIUTADA
(
  CODCTD 
)
ENABLE;

ALTER TABLE ESD_ANY
ADD CONSTRAINT FK_ESD_ANY_LOCALITAT FOREIGN KEY
(
  EST_CODEST 
, REG_CODREG 
, LOC_CODLOC 
)
REFERENCES LOCALITAT
(
  EST_CODEST 
, REG_CODREG 
, CODLOC 
)
ENABLE;

ALTER TABLE ESD_ANY
ADD CONSTRAINT FK_ESD_ANY_VOTACIO FOREIGN KEY
(
  VOT_CODVOT 
)
REFERENCES VOTACIO
(
  CODVOT 
)
ENABLE;

ALTER TABLE ESD_LOCALITAT_ANY
ADD CONSTRAINT RF_ELA_LOC FOREIGN KEY
(
  EST_CODEST 
, REG_CODREG 
, LOC_CODLOC 
)
REFERENCES LOCALITAT
(
  EST_CODEST 
, REG_CODREG 
, CODLOC 
)
ENABLE;

ALTER TABLE ESD_VOTACIO
ADD CONSTRAINT ESD_VOTACIO_ESTAT_FK FOREIGN KEY
(
  EST_CODEST_MAXPRTCTD 
)
REFERENCES ESTAT
(
  CODEST 
)
ENABLE;

ALTER TABLE ESD_VOTACIO
ADD CONSTRAINT ESD_VOTACIO_VOTACIO_FK FOREIGN KEY
(
  VOT_CODVOT_MAXPRT 
)
REFERENCES VOTACIO
(
  CODVOT 
)
ENABLE;

ALTER TABLE ESD_VOTACIO
ADD CONSTRAINT ESD_VOTACIO_VOTACIO_FK1 FOREIGN KEY
(
  VOT_CODVOT_MAXDIF 
)
REFERENCES VOTACIO
(
  CODVOT 
)
ENABLE;

ALTER TABLE ESD_VOTACIO
ADD CONSTRAINT ESD_VOTACIO_VOTACIO_FK2 FOREIGN KEY
(
  VOT_CODVOT_MINDIF 
)
REFERENCES VOTACIO
(
  CODVOT 
)
ENABLE;

ALTER TABLE ESD_ESTAT_ANY
ADD CONSTRAINT ESD_ESTAT_ANY_ESTAT_FK FOREIGN KEY
(
  EST_CODEST 
)
REFERENCES ESTAT
(
  CODEST 
)
ENABLE;

COMMENT ON COLUMN ESD_CENS_ANY.CEN_CODCEN IS 'Cens (Codi)';

COMMENT ON COLUMN ESD_CENS_ANY.NUMANY IS 'Estadstics Cens Any (Any)';

COMMENT ON COLUMN ESD_CENS_ANY.PRCPRTMIG IS 'Estadsitcs Cens Any (Percentatge mig de participaci)';

COMMENT ON COLUMN ESD_CENS_ANY.NUMVOTCEN IS 'Estadsitcs Cens Any (Nmero de votacions del cens)';

COMMENT ON COLUMN ESD_CIUTADA.NUMPRTVOT IS 'Estadstiques ciutad (Nmero de votacions)';

COMMENT ON COLUMN ESD_ANY.NUMANY IS 'Estadsitcs Any (Any)';

COMMENT ON COLUMN ESD_ANY.PRCCTDNOV IS 'Estadsitcs Any (Percentatge de ciutadans que mai han votat)';

COMMENT ON COLUMN ESD_ANY.NUMMAXVOT IS 'Estadstiques any (Nmero mxim de vots localitat)';

COMMENT ON COLUMN ESD_LOCALITAT_ANY.NUMANY IS 'Estadstic localitat anual (Any)';

COMMENT ON COLUMN ESD_LOCALITAT_ANY.NUMVOT IS 'Estadstic localitat anual (Vots emesos)';

COMMENT ON COLUMN ESD_VOTACIO.VOT_CODVOT_MAXPRT IS 'Estadistics votaci (Votaci amb ms participaci)';

COMMENT ON COLUMN ESD_VOTACIO.VOT_CODVOT_MAXDIF IS 'Estadstics votaci (Votaci amb mxima diferencia entre la opci ms votada i la menys votada)';

COMMENT ON COLUMN ESD_VOTACIO.VOT_CODVOT_MINDIF IS 'Estadstics votaci (Votaci amb mnima diferencia entre la opci ms votada i la menys votada)';

COMMENT ON COLUMN ESD_VOTACIO.NUMMAXPRTCTD IS 'Estadstics votaci (Nmero mxim de votacions en las que ha participat una mateixa persona)';

COMMENT ON COLUMN ESD_ESTAT_ANY.EST_CODEST IS 'Estat (Codi)';

COMMENT ON COLUMN ESD_ESTAT_ANY.NUMANY IS 'Estadsitcs Estat Any (Any)';

COMMENT ON COLUMN ESD_ESTAT_ANY.NUMVOTFIN IS 'Estadsitcs Estat Any (Votacions finalitzades)';

CREATE INDEX ID_ESD_CENS_ANY ON ESD_CENS_ANY (NUMANY);

CREATE INDEX IF_ESD_ANY_LOC ON ESD_ANY (EST_CODEST, REG_CODREG, LOC_CODLOC);

CREATE INDEX IF_ESD_ANY_VOTACIO ON ESD_ANY (VOT_CODVOT);

CREATE INDEX IF_ESD_ESTAT_ANY ON ESD_ESTAT_ANY (NUMANY);

exit
/
