create or replace package body CENS_PKG is

  -- Comprova si el cens indicat existeix.
  --
  -- %param P_CODCEN Cens a comprovar
  --
  -- %return Indicador de si el cens existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION CENS_EXISTEIX (P_CODCEN    CENS.CODCEN%TYPE
                         ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from CENS cen
                      where cen.CODCEN = P_CODCEN);

    return (vExisteix > 0);
  END;

  -- Comprova si existeix un cens del tipus indicat seogns
  -- si es un ces global, estatal, regional o local.
  -- %param P_TIPCEN      Tipus de cens
  --                        {*} GLB global
  --                        {*} EST estatal
  --                        {*} REG regional
  --                        {*} LOC local
  -- %param P_CODEST      Codi destat. Obligatori per a tots els tipus de censos a excepci del global.
  -- %param P_CODREG      Codi de regi. Obligatori per els tipus de censos regional i local.
  -- %param P_CODLOC      Codi de localitat. Obligatori per els censos locals.
  --
  -- %return Indicador de si el cens existeix o no.
  --           {*}  TRUE   Exsiteix
  --           {*}  FALSE  No existeix
  FUNCTION EXISTEIX_CENS_TIPUS (P_TIPCEN    CENS.TIPCEN%TYPE,
                                P_CODEST    CENS.EST_CODEST%TYPE := null,
                                P_CODREG    CENS.REG_CODREG%TYPE := null,
                                P_CODLOC    CENS.LOC_CODLOC%TYPE := null
                               ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from CENS cen
                      where cen.TIPCEN = P_TIPCEN and
                           (P_CODEST is null or cen.EST_CODEST = P_CODEST) and
                           (P_CODREG is null or cen.REG_CODREG = P_CODREG) and
                           (P_CODLOC is null or cen.LOC_CODLOC = P_CODLOC));

    return (vExisteix > 0);
  END;

  -- Obt el cens global.
  --
  -- %return Codi del cens global. Si no existeix, retorna el valor null.
  FUNCTION OBTENIR_CENS_GLOBAL RETURN CENS.CODCEN%TYPE IS

    Cursor cCen is
      Select cen.CODCEN
        from CENS cen
        where cen.TIPCEN = cGLOBAL;

    vCodCen    CENS.CODCEN%TYPE;
  BEGIN
    Open cCen;
    Fetch cCen into vCodCen;
    Close cCen;

    return vCodCen;
  END;

  -- Obt el cens associat a l'estat indicat.
  --
  -- %param P_CODEST      Codi destat.
  --
  -- %return Codi del cens associat a l'esat. Si no existeix, retorna el valor null.
  FUNCTION OBTENIR_CENS_ESTATAL (P_CODEST    CENS.EST_CODEST%TYPE
                                ) RETURN CENS.CODCEN%TYPE IS
    Cursor cCen is
      Select cen.CODCEN
        from CENS cen
        where cen.TIPCEN = cESTATAL and
              cen.EST_CODEST = P_CODEST;

    vCodCen    CENS.CODCEN%TYPE;
  BEGIN
    Open cCen;
    Fetch cCen into vCodCen;
    Close cCen;

    return vCodCen;
  END;

  -- Obt el cens associat a la regi indicada.
  --
  -- %param P_CODEST      Codi destat.
  -- %param P_CODREG      Codi de regi.
  --
  -- %return Codi del cens associat a la regi. Si no existeix,
  --         retorna el valor null.
  FUNCTION OBTENIR_CENS_REGIONAL (P_CODEST    CENS.EST_CODEST%TYPE,
                                  P_CODREG    CENS.REG_CODREG%TYPE
                                 ) RETURN CENS.CODCEN%TYPE IS
    Cursor cCen is
      Select cen.CODCEN
        from CENS cen
        where cen.TIPCEN = cREGIONAL and
              cen.EST_CODEST = P_CODEST and
              cen.REG_CODREG = P_CODREG;

    vCodCen    CENS.CODCEN%TYPE;
  BEGIN
    Open cCen;
    Fetch cCen into vCodCen;
    Close cCen;

    return vCodCen;
  END;

  -- Obt el cens associat a la localitat indicada.
  --
  -- %param P_CODEST      Codi destat.
  -- %param P_CODREG      Codi de regi.
  -- %param P_CODLOC      Codi de localitat.
  --
  -- %return Codi del cens associat a la localitat. Si no existeix,
  --         retorna el valor null.
  FUNCTION OBTENIR_CENS_LOCAL (P_CODEST    CENS.EST_CODEST%TYPE,
                               P_CODREG    CENS.REG_CODREG%TYPE,
                               P_CODLOC    CENS.LOC_CODLOC%TYPE
                              ) RETURN CENS.CODCEN%TYPE IS
    Cursor cCen is
      Select cen.CODCEN
        from CENS cen
        where cen.TIPCEN = cLOCAL and
              cen.EST_CODEST = P_CODEST and
              cen.REG_CODREG = P_CODREG and
              cen.LOC_CODLOC = P_CODLOC;

    vCodCen    CENS.CODCEN%TYPE;
  BEGIN
    Open cCen;
    Fetch cCen into vCodCen;
    Close cCen;

    return vCodCen;
  END;


  -- Obt el cens de nivell superior
  --
  -- %param P_TIPCEN      Tipus de cens
  --                        {*} GLB global
  --                        {*} EST estatal
  --                        {*} REG regional
  --                        {*} LOC local
  -- %param P_CODEST      Codi destat.
  -- %param P_CODREG      Codi de regi
  --
  -- %return Indicador de si el cens existeix o no.
  --           {*}  TRUE   Exsiteix
  --           {*}  FALSE  No existeix
  FUNCTION OBTENIR_CENS_SUPERIOR (P_TIPCEN    CENS.TIPCEN%TYPE,
                                  P_CODEST    CENS.EST_CODEST%TYPE := null,
                                  P_CODREG    CENS.REG_CODREG%TYPE := null
                                 ) RETURN CENS.CODCEN%TYPE IS

    vCodCen    CENS.CODCEN%TYPE;
  BEGIN
    -- Segons el tipus de cens, busquem el cens superior que l'englova
    Case P_TIPCEN
      when cGLOBAL then
        vCodCen := null; -- No en t
      when cESTATAL then
        vCodCen := OBTENIR_CENS_GLOBAL;
      when cREGIONAL then
        vCodCen := OBTENIR_CENS_ESTATAL (P_CODEST);
      when cLOCAL then
        vCodCen := OBTENIR_CENS_REGIONAL (P_CODEST, P_CODREG);
      else
        vCodCen := null;
    end case;

    return vCodCen;
  END;

  -- Dona dalta un cens.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi
  -- 101           Sha dinformar el tipus
  -- 102           Tipus de cens no vlid
  -- 103           Ja existeix un cens amb aquest codi
  -- 104           Noms hi pot haver un cens global
  -- 105           Lestat s obligatori per al tipus de cens
  -- 106           La regi s obligatria per al tipus de cens
  -- 107           La localitat s obligatoria per al tipus de cens
  -- 108           L'estat no existeix
  -- 109           La regi no existeix
  -- 110           La localitat no existeix
  -- 111           Ja existeix un cens estatal per lestat
  -- 112           Ja existeix un cens regional per a la regi
  -- 113           Ja existeix un cens local per a la localitat
  -- 111           No sha trobat el cens superior
  -- 900           Error inesperat
  --
  -- %param P_CODCEN      Codi del ciutad a donar de baixa
  -- %param P_TIPCEN      Tipus de cens
  --                        {*} GLB global
  --                        {*} EST estatal
  --                        {*} REG regional
  --                        {*} LOC local
  -- %param P_CODEST      Codi destat. Obligatori per a tots els tipus de censos a excepci del global.
  -- %param P_CODREG      Codi de regi. Obligatori per els tipus de censos regional i local.
  -- %param P_CODLOC      Codi de localitat. Obligatori per els censos locals.
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_ALTA (P_CODCEN        CENS.CODCEN%TYPE,
                       P_TIPCEN        CENS.TIPCEN%TYPE,
                       P_CODEST        CENS.EST_CODEST%TYPE,
                       P_CODREG        CENS.REG_CODREG%TYPE,
                       P_CODLOC        CENS.LOC_CODLOC%TYPE,
                       P_RSP       OUT VARCHAR2
                      ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    vCodCenSup    CENS.CODCEN%TYPE;
    rCen          CENS%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CENS_ALTA', 'Alta d''un cens');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCEN        : ' || P_CODCEN);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_TIPCEN        : ' || P_TIPCEN);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Busquem el codi del cens superior
    vCodCenSup := OBTENIR_CENS_SUPERIOR (P_TIPCEN, P_CODEST, P_CODREG);

    -- Comprovacions prvies
    If P_CODCEN is null then
      P_RSP := '100-El CODI del cens s obligatori';
    elsif P_TIPCEN is null then
      P_RSP := '101-El TIPUS del cens s obligatori';
    elsif P_TIPCEN not in (cGLOBAL, cESTATAL, cREGIONAL, cLOCAL) then
      P_RSP := '102-TIPUS de cens no vlid';
    elsif CENS_EXISTEIX (P_CODCEN) then
      P_RSP := '103-Ja existeix un cens amb el CODI ' || P_CODCEN;
    elsif P_TIPCEN = cGLOBAL and EXISTEIX_CENS_TIPUS (cGLOBAL) then
      P_RSP := '104-Noms hi pot haver un cens global';
    elsif P_TIPCEN in (cESTATAL, cREGIONAL, cLOCAL) and P_CODEST is null then
      P_RSP := '105-L''esat s obligatori per al tipus de cens';
    elsif P_TIPCEN in (cREGIONAL, cLOCAL) and P_CODREG is null then
      P_RSP := '106-La regi s obligatoria per al tipus de cens';
    elsif P_TIPCEN = cLOCAL and P_CODLOC is null then
      P_RSP := '107-La localitat s obligatoria per al tipus de cens';
    elsif P_TIPCEN = cESTATAL and not GEOGRAFIC_PKG.ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '108-No existeix cap estat amb CODI ' || P_CODEST;
    elsif P_TIPCEN = cREGIONAL and not GEOGRAFIC_PKG.REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '109-No existeix cap regi amb CODI ' || P_CODREG;
    elsif P_TIPCEN = cLOCAL and not GEOGRAFIC_PKG.LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '110-No existeix cap la localitat amb CODI ' || P_CODLOC;
    elsif P_TIPCEN = cESTATAL and EXISTEIX_CENS_TIPUS (cESTATAL, P_CODEST) then
      P_RSP := '111-Ja existeix un cens estatal per a l''estat amb CODI ' || P_CODEST;
    elsif P_TIPCEN = cREGIONAL and EXISTEIX_CENS_TIPUS (cREGIONAL, P_CODEST, P_CODREG) then
      P_RSP := '112-Ja existeix un cens regional per a la regi amb CODI ' || P_CODREG;
    elsif P_TIPCEN = cLOCAL and EXISTEIX_CENS_TIPUS (cLOCAL, P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '113-Ja existeix un cens local per a la localitat amb CODI ' || P_CODLOC;
    elsif P_TIPCEN != cGLOBAL and vCodCenSup is null then
      P_RSP := '114-No s''ha trobat el cens superior';
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rCen.CODCEN     := P_CODCEN;
      rCen.TIPCEN     := P_TIPCEN;
      rCen.CEN_CODCEN := vCodCenSup;
      rCen.EST_CODEST := P_CODEST;
      rCen.REG_CODREG := P_CODREG;
      rCen.LOC_CODLOC := P_CODLOC;

      Insert into CENS
        values rCen;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  PROCEDURE BAIXA_CENSOS_INFERIORS (P_CODCEN    CENS.CODCEN%TYPE
                                   ) IS
  BEGIN
    For rCenInf in (Select cen.CODCEN
                     from CENS cen
                     where cen.CEN_CODCEN = P_CODCEN
                     for update nowait) loop


      -- Donem de baixa els possibles censos inferiors
      BAIXA_CENSOS_INFERIORS (rCenInf.CODCEN);

      -- Esborrem el cens
      Delete from CENS cen
        where cen.CODCEN = rCenInf.CODCEN;
    end loop;
  END;

  -- Dona de baixa un cens. Si t altres censos de nivells inferiors, tamb els dona de baixa.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el cens
  -- 101	         El cens no existeix
  -- 900	         Error inesperat
  --
  -- %param P_CODCEN      Codi de cens
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_BAIXA (P_CODCEN        CENS.CODCEN%TYPE,
                        P_RSP       OUT VARCHAR2
                       ) IS

    Cursor cCen is
      Select 1
        from CENS cen
        where cen.CODCEN = P_CODCEN
        for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CENS_BAIXA', 'Dona de baixa un cens');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCEN        : ' || P_CODCEN);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCEN is null then
      P_RSP := '100-El CODI del cens s obligatori';
    elsif not CENS_EXISTEIX (P_CODCEN) then
      P_RSP := '103-No existeix cap cens amb el CODI ' || P_CODCEN;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      BAIXA_CENSOS_INFERIORS (P_CODCEN);

      Open cCen;
      Close cCen;

      Delete CENS cen
        where cen.CODCEN = P_CODCEN;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Obt el nombre de ciutadans del cens que tenen dret a votar (majors
  -- de 18 anys en una data concreta).
  --
  -- %param P_CODCEN     Codi del cens
  -- %param P_DATCTL     Data de control
  --
  -- %return Nombre de ciutadans amb dret a votar
  FUNCTION NOMBRE_CIUTADANS_CENS (P_CODCEN    CENS.CODCEN%TYPE,
                                  P_DATCTL    DATE
                                 ) RETURN NUMBER IS

    Cursor cCen is
      Select cen.*
        from CENS cen
        where cen.CODCEN = P_CODCEN;

    rCen       CENS%ROWTYPE;
    vNumCtd    number;
  BEGIN
    -- Obtenim dades del cens
    Open cCen;
    Fetch cCen into rCen;
    Close cCen;

    -- Segons el tipus de cens, calculem d'una forma o altre
    -- Cens global: tots els ciutadans majors de 18 anys en el moment de
    --              tancar la votaci
    If rCen.TIPCEN = CENS_PKG.cGLOBAL then

      Select nvl (count(1), 0)
        into vNumCtd
        from CIUTADA ctd
        where ((trunc (P_DATCTL) - trunc (ctd.DATNAC)) / 365) >= 18;

    -- Cens estatal: tots els ciutadans de l'estat majors de 18 anys en
    --               el moment de tancar la votaci
    elsif rCen.TIPCEN = CENS_PKG.cESTATAL then

      Select nvl (count(1), 0)
        into vNumCtd
        from CIUTADA ctd
        where ctd.EST_CODEST = rCen.EST_CODEST and
              ((trunc (P_DATCTL) - trunc (ctd.DATNAC)) / 365) >= 18;

    -- Cens regional: tots els ciutadans de la regi majors de 18 anys en
    --                el moment de tancar la votaci
    elsif rCen.TIPCEN = CENS_PKG.cREGIONAL then

      Select nvl (count(1), 0)
        into vNumCtd
        from CIUTADA ctd
        where ctd.EST_CODEST = rCen.EST_CODEST and
              ctd.REG_CODREG = rCen.REG_CODREG and
              ((trunc (P_DATCTL) - trunc (ctd.DATNAC)) / 365) >= 18;

    -- Cens local: tots els ciutadans de la localitat majors de 18 anys en
    --             el moment de tancar la votaci
    elsif rCen.TIPCEN = CENS_PKG.cLOCAL then

      Select nvl (count(1), 0)
        into vNumCtd
        from CIUTADA ctd
        where ctd.EST_CODEST = rCen.EST_CODEST and
              ctd.REG_CODREG = rCen.REG_CODREG and
              ctd.LOC_CODLOC = rCen.LOC_CODLOC and
              ((trunc (P_DATCTL) - trunc (ctd.DATNAC)) / 365) >= 18;
    end if;

    return vNumCtd;
  END;

  FUNCTION CIUTADA_PERTANY_A_CENS (P_CODCTD    CIUTADA.CODCTD%TYPE,
                                   P_CODCEN    CENS.CODCEN%TYPE
                                  ) RETURN NUMBER IS

    Cursor cCen is
      Select cen.TIPCEN, cen.EST_CODEST, cen.REG_CODREG, cen.LOC_CODLOC
        from CENS cen
        where cen.CODCEN = P_CODCEN;

    Cursor cCtd is
      Select ctd.EST_CODEST, ctd.REG_CODREG, ctd.LOC_CODLOC
        from CIUTADA ctd
        where ctd.CODCTD = P_CODCTD;

    rCen             cCen%ROWTYPE;
    rCtd             cCtd%ROWTYPE;
    vPertanyACens    number;
  BEGIN
    vPertanyACens := 0;

    -- Si s el cens global, segur que hi pertany.
    If P_CODCEN = OBTENIR_CENS_GLOBAL then
      vPertanyACens := 1;

    -- Sin, segons el tipus de cens, comprovem unes dades
    -- o altres
    else
      -- Dades del cens
      Open cCen;
      Fetch cCen into rCen;
      Close cCen;

      -- Dades del ciutada
      Open cCtd;
      Fetch cCtd into rCtd;
      Close cCtd;

      If rCen.TIPCEN = cESTATAL and
         rCen.EST_CODEST = rCtd.EST_CODEST then

        vPertanyACens := 1;

      elsif rCen.TIPCEN = cREGIONAL and
            rCen.EST_CODEST = rCtd.EST_CODEST and
            rCen.REG_CODREG = rCtd.REG_CODREG then

        vPertanyACens := 1;

      elsif rCen.TIPCEN = cLOCAL and
            rCen.EST_CODEST = rCtd.EST_CODEST and
            rCen.REG_CODREG = rCtd.REG_CODREG and
            rCen.LOC_CODLOC = rCtd.LOC_CODLOC then

        vPertanyACens := 1;
      end if;
    end if;

    return vPertanyACens;
  END;

end;
/

exit
/
