create or replace package CENS_PKG is

  -- Tipus de censos
  cGLOBAL    constant varchar2(3) := 'GLB';
  cESTATAL   constant varchar2(3) := 'EST';
  cREGIONAL  constant varchar2(3) := 'REG';
  cLOCAL     constant varchar2(3) := 'LOC';

  -- Obt el cens global.
  --
  -- %return Codi del cens global. Si no existeix, retorna el valor null.
  FUNCTION OBTENIR_CENS_GLOBAL RETURN CENS.CODCEN%TYPE;

  -- Obt el cens associat a l'estat indicat.
  --
  -- %param P_CODEST      Codi destat.
  --
  -- %return Codi del cens associat a l'esat. Si no existeix, retorna el valor null.
  FUNCTION OBTENIR_CENS_ESTATAL (P_CODEST    CENS.EST_CODEST%TYPE
                                ) RETURN CENS.CODCEN%TYPE;

  -- Obt el cens associat a la regi indicada.
  --
  -- %param P_CODEST      Codi destat.
  -- %param P_CODREG      Codi de regi.
  --
  -- %return Codi del cens associat a la regi. Si no existeix,
  --         retorna el valor null.
  FUNCTION OBTENIR_CENS_REGIONAL (P_CODEST    CENS.EST_CODEST%TYPE,
                                  P_CODREG    CENS.REG_CODREG%TYPE
                                 ) RETURN CENS.CODCEN%TYPE;

  -- Obt el cens associat a la localitat indicada.
  --
  -- %param P_CODEST      Codi destat.
  -- %param P_CODREG      Codi de regi.
  -- %param P_CODLOC      Codi de localitat.
  --
  -- %return Codi del cens associat a la localitat. Si no existeix,
  --         retorna el valor null.
  FUNCTION OBTENIR_CENS_LOCAL (P_CODEST    CENS.EST_CODEST%TYPE,
                               P_CODREG    CENS.REG_CODREG%TYPE,
                               P_CODLOC    CENS.LOC_CODLOC%TYPE
                              ) RETURN CENS.CODCEN%TYPE;

  -- Comprova si el cens indicat existeix.
  --
  -- %param P_CODCEN Cens a comprovar
  --
  -- %return Indicador de si el cens existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION CENS_EXISTEIX (P_CODCEN    CENS.CODCEN%TYPE
                         ) RETURN BOOLEAN;

  -- Dona dalta un cens.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi
  -- 101           Sha dinformar el tipus
  -- 102           Tipus de cens no vlid
  -- 103           Ja existeix un cens amb aquest codi
  -- 104           Noms hi pot haver un cens global
  -- 105           Lestat s obligatori per al tipus de cens
  -- 106           La regi s obligatria per al tipus de cens
  -- 107           La localitat s obligatoria per al tipus de cens
  -- 108           Ja existeix un cens estatal per lestat
  -- 109           Ja existeix un cens regional per a la regi
  -- 110           Ja existeix un cens local per a la localitat
  -- 111           No sha trobat el cens superior
  -- 900           Error inesperat
  --
  -- %param P_CODCEN      Codi del ciutad a donar de baixa
  -- %param P_TIPCEN      Tipus de cens
  --                        {*} GLB global
  --                        {*} EST estatal
  --                        {*} REG regional
  --                        {*} LOC local
  -- %param P_CODEST      Codi destat. Obligatori per a tots els tipus de censos a excepci del global.
  -- %param P_CODREG      Codi de regi. Obligatori per els tipus de censos regional i local.
  -- %param P_CODLOC      Codi de localitat. Obligatori per els censos locals.
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el valor OK, sin retorna un codi derror  seguit de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_ALTA (P_CODCEN        CENS.CODCEN%TYPE,
                       P_TIPCEN        CENS.TIPCEN%TYPE,
                       P_CODEST        CENS.EST_CODEST%TYPE,
                       P_CODREG        CENS.REG_CODREG%TYPE,
                       P_CODLOC        CENS.LOC_CODLOC%TYPE,
                       P_RSP       OUT VARCHAR2);

  -- Dona de baixa un cens. Si t altres censos de nivells inferiors, tamb els dona de baixa.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el cens
  -- 101	         El cens no existeix
  -- 900	         Error inesperat
  --
  -- %param P_CODCEN      Codi de cens
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_BAIXA (P_CODCEN        CENS.CODCEN%TYPE,
                        P_RSP       OUT VARCHAR2);

  -- Obt el nombre de ciutadans del cens que tenen dret a votar (majors
  -- de 18 anys en una data concreta).
  --
  -- %param P_CODCEN     Codi del cens
  -- %param P_DATCTL     Data de control
  --
  -- %return Nombre de ciutadans amb dret a votar
  FUNCTION NOMBRE_CIUTADANS_CENS (P_CODCEN    CENS.CODCEN%TYPE,
                                  P_DATCTL    DATE
                                 ) RETURN NUMBER;

  FUNCTION CIUTADA_PERTANY_A_CENS (P_CODCTD    CIUTADA.CODCTD%TYPE,
                                   P_CODCEN    CENS.CODCEN%TYPE
                                  ) RETURN NUMBER;
end;
/

exit
/
