create or replace package body CIUTADA_PKG is

  -- Comprova l'existncia d'un ciutad.
  --
  -- %param P_CODCTD  Ciutad a comprovar
  --
  -- %return Indicador sobre si existeix un ciutad amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION CIUTADA_EXISTEIX (P_CODCTD    CIUTADA.CODCTD%TYPE
                            ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from CIUTADA ctd
                      where ctd.CODCTD = P_CODCTD);

    return (vExisteix > 0);
  END;

  -- Comprova si el ciuta indicat t dades asociades
  -- Dades a comprovar:
  --
  -- %param P_CODCTD  Ciutad a comprovar
  --
  -- %return Indicador sobre si l'estat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION CIUTADA_TE_DADES_ASOCIADES (P_CODCTD    CIUTADA.CODCTD%TYPE
                                      ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    vExisteix := 0;
    /*
    Select count(1)
      into vExisteix
      from dual
      where exists (-- Regions
                    Select 1
                      from REGIO reg
                      where reg.EST_CODEST = P_CODEST);
    */

    return (vExisteix > 0);
  END;

  -- Dona dalta un ciutad.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            Sha dinformar la data de naixement
  -- 102            Sha dinformar lestat
  -- 103            Sha dinformar la regi
  -- 104            Sha dinformar la localitat
  -- 105            Lestat no existeix
  -- 106            La regi no existeix
  -- 107            La localitat no existeix
  -- 108            Ja exiteix un ciutad amb el codi indicat
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_DATNAC  Data de naixement
  -- %param P_CODEST  Codi de lestat de la localitat del ciutad
  -- %param P_CODREG  Codi de la regi de la localitat del ciutad
  -- %param P_CODLOC  Codi de la localitat del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_ALTA (P_CODCTD    CIUTADA.CODCTD%TYPE,
                          P_DATNAC    CIUTADA.DATNAC%TYPE,
                          P_CODEST    CIUTADA.EST_CODEST%TYPE,
                          P_CODREG    CIUTADA.REG_CODREG%TYPE,
                          P_CODLOC    CIUTADA.LOC_CODLOC%TYPE,
                          P_RSP       OUT VARCHAR2
                         ) IS

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rCtd       CIUTADA%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CIUTADA_ALTA', 'Alta d''un ciutad');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATNAC        : ' || P_DATNAC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CODI del ciutad s obligatori';
    elsif P_DATNAC is null then
      P_RSP := '101-La DATA NAIXAMENT del ciutad s obligatria';
    elsif P_CODEST is null then
      P_RSP := '102-El CODI ESTAT s obligatori';
    elsif P_CODREG is null then
      P_RSP := '103-El CODI REGI s obligatori';
    elsif P_CODLOC is null then
      P_RSP := '104-El CODI LOCALITAT s obligatori';
    elsif not GEOGRAFIC_PKG.ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '105-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not GEOGRAFIC_PKG.REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '106-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif not GEOGRAFIC_PKG.LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '107-No existeix cap localitat amb el CODI ' || P_CODLOC;
    elsif CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '108-Ja existeix un ciutad amb el CODI ' || P_CODCTD;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rCtd.CODCTD     := P_CODCTD;
      rCtd.DATNAC     := P_DATNAC;
      rCtd.EST_CODEST := P_CODEST;
      rCtd.REG_CODREG := P_CODREG;
      rCtd.LOC_CODLOC := P_CODLOC;

      Insert into CIUTADA
        values rCtd;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades dun ciutad (data naixement i localitat).
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            Sha dinformar la data de naixement
  -- 102            Sha dinformar lestat
  -- 103            Sha dinformar la regi
  -- 104            Sha dinformar la localitat
  -- 105            Lestat no existeix
  -- 106            La regi no existeix
  -- 107            La localitat no existeix
  -- 108            El ciutad no existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_DATNAC  Data de naixement
  -- %param P_CODEST  Codi de lestat de la localitat del ciutad
  -- %param P_CODREG  Codi de la regi de la localitat del ciutad
  -- %param P_CODLOC  Codi de la localitat del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_MODIFICACIO (P_CODCTD    CIUTADA.CODCTD%TYPE,
                                 P_DATNAC    CIUTADA.DATNAC%TYPE,
                                 P_CODEST    CIUTADA.EST_CODEST%TYPE,
                                 P_CODREG    CIUTADA.REG_CODREG%TYPE,
                                 P_CODLOC    CIUTADA.LOC_CODLOC%TYPE,
                                 P_RSP       OUT VARCHAR2
                                ) IS

    Cursor cCtd is
      Select ctd.*
        from CIUTADA ctd
        where ctd.CODCTD = P_CODCTD
        for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rCtd       CIUTADA%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CIUTADA_MODIFICACIO', 'Modifica dades d''un ciutad');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATNAC        : ' || P_DATNAC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CODI del ciutad s obligatori';
    elsif P_DATNAC is null then
      P_RSP := '101-La DATA NAIXAMENT del ciutad s obligatria';
    elsif P_CODEST is null then
      P_RSP := '102-El CODI ESTAT s obligatori';
    elsif P_CODREG is null then
      P_RSP := '103-El CODI REGI s obligatori';
    elsif P_CODLOC is null then
      P_RSP := '104-El CODI LOCALITAT s obligatori';
    elsif not GEOGRAFIC_PKG.ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '105-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not GEOGRAFIC_PKG.REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '106-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif not GEOGRAFIC_PKG.LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '107-No existeix cap localitat amb el CODI ' || P_CODLOC;
    elsif not CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '108-No existeix cap ciutad amb el CODI ' || P_CODCTD;
    end if;

    -- s tot correcte, realitzem la modificaci
    If P_RSP = 'OK' then

      Open cCtd;
      Fetch cCtd into rCtd;

      rCtd.DATNAC     := P_DATNAC;
      rCtd.EST_CODEST := P_CODEST;
      rCtd.REG_CODREG := P_CODREG;
      rCtd.LOC_CODLOC := P_CODLOC;

      Update CIUTADA
        set row = rCtd
        where current of cCtd;

      Close cCtd;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa un ciutad.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            El ciutad no existeix
  -- 102            El ciutad t dades asociades. No es permet la seva baixa
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_BAIXA (P_CODCTD    CIUTADA.CODCTD%TYPE,
                           P_RSP       OUT VARCHAR2
                          ) IS

    Cursor cCtd is
      Select 1
        from CIUTADA ctd
        where ctd.CODCTD = P_CODCTD
        for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CIUTADA_BAIXA', 'Dona de baixa un ciutad');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CODI del ciutad s obligatori';
    elsif not CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '101-No existeix cap ciutad amb el CODI ' || P_CODCTD;
    elsif CIUTADA_TE_DADES_ASOCIADES (P_CODCTD) then
      P_RSP := '102-El ciutad t dades asociades. No es permet la seva baixa.';
    end if;

    -- s tot correcte, realitzem la modificaci
    If P_RSP = 'OK' then

      Open cCtd;
      Close cCtd;

      Delete from CIUTADA ctd
        where ctd.CODCTD = P_CODCTD;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;
end;
/

exit
/
