create or replace package CIUTADA_PKG is

  -- Comprova l'existncia d'un ciutad.
  --
  -- %param P_CODCTD  Ciutad a comprovar
  --
  -- %return Indicador sobre si existeix un ciutad amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION CIUTADA_EXISTEIX (P_CODCTD    CIUTADA.CODCTD%TYPE
                            ) RETURN BOOLEAN;

  -- Comprova si el ciuta indicat t dades asociades
  -- Dades a comprovar:
  --
  -- %param P_CODCTD  Ciutad a comprovar
  --
  -- %return Indicador sobre si l'estat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION CIUTADA_TE_DADES_ASOCIADES (P_CODCTD    CIUTADA.CODCTD%TYPE
                                      ) RETURN BOOLEAN;

  -- Dona dalta un ciutad.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            Sha dinformar la data de naixement
  -- 102            Sha dinformar lestat
  -- 103            Sha dinformar la regi
  -- 104            Sha dinformar la localitat
  -- 105            Lestat no existeix
  -- 106            La regi no existeix
  -- 107            La localitat no existeix
  -- 108            Ja exiteix un ciutad amb el codi indicat
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_DATNAC  Data de naixement
  -- %param P_CODEST  Codi de lestat de la localitat del ciutad
  -- %param P_CODREG  Codi de la regi de la localitat del ciutad
  -- %param P_CODLOC  Codi de la localitat del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_ALTA (P_CODCTD    CIUTADA.CODCTD%TYPE,
                          P_DATNAC    CIUTADA.DATNAC%TYPE,
                          P_CODEST    CIUTADA.EST_CODEST%TYPE,
                          P_CODREG    CIUTADA.REG_CODREG%TYPE,
                          P_CODLOC    CIUTADA.LOC_CODLOC%TYPE,
                          P_RSP       OUT VARCHAR2);

  -- Modifica les dades dun ciutad (data naixement i localitat).
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            Sha dinformar la data de naixement
  -- 102            Sha dinformar lestat
  -- 103            Sha dinformar la regi
  -- 104            Sha dinformar la localitat
  -- 105            Lestat no existeix
  -- 106            La regi no existeix
  -- 107            La localitat no existeix
  -- 108            El ciutad no existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_DATNAC  Data de naixement
  -- %param P_CODEST  Codi de lestat de la localitat del ciutad
  -- %param P_CODREG  Codi de la regi de la localitat del ciutad
  -- %param P_CODLOC  Codi de la localitat del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_MODIFICACIO (P_CODCTD    CIUTADA.CODCTD%TYPE,
                                 P_DATNAC    CIUTADA.DATNAC%TYPE,
                                 P_CODEST    CIUTADA.EST_CODEST%TYPE,
                                 P_CODREG    CIUTADA.REG_CODREG%TYPE,
                                 P_CODLOC    CIUTADA.LOC_CODLOC%TYPE,
                                 P_RSP       OUT VARCHAR2);

  -- Dona de baixa un ciutad.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar el codi
  -- 101            El ciutad no existeix
  -- 102            El ciutad t dades asociades. No es permet la seva baixa
  -- 900            Error inesperat
  --
  --
  -- %param P_CODCTD  Codi del ciutad
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_BAIXA (P_CODCTD    CIUTADA.CODCTD%TYPE,
                           P_RSP       OUT VARCHAR2);
end;
/

exit
/
