create or replace package body CONSULTES_PKG is

  -- Donat un pas, obt el llistat de les votacions que s'hi han produt.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi de l'estat
  -- 101           L'estat no existeix
  -- 900           Error inesperat
  --
  -- %param P_CODESt      Codi d'estat
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  -- %param P_LLISTAT OUT Cursor referenciat que cont les dades del llistat.
  PROCEDURE VOTACIONS_PER_ESTAT (P_CODEST        ESTAT.CODEST%TYPE,
                                 P_RSP       OUT VARCHAR2,
                                 P_LLISTAT   OUT SYS_REFCURSOR
                                 ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIONS_PER_ESTAT', 'Obt la llista de votacions d''un estat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI d''estat s obligatori';
    elsif not GEOGRAFIC_PKG.ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '101-No existeix cap estat amb el CODI ' || P_CODEST;
    end if;

    -- s tot correcte, realitzem el llistat
    If P_RSP = 'OK' then

      If P_LLISTAT%ISOPEN then
        Close P_LLISTAT;
      end if;
      
      Open P_LLISTAT for
         Select vot.INFTITVOT,
                vot.DATINIVOT,
                vot.DATFINVOT,
                decode (vot.ESTVOT, 'EDI', 'EN EDICI',
                                    'PDT', 'PENDENT',
                                    'INI', 'INICIADA',
                                    'FIN', 'FINALITZADA')  ESTVOT,
               (Select sum (CENS_PKG.NOMBRE_CIUTADANS_CENS (cen.CODCEN, vot.DATFINVOT))
                                from CENS_VOTACIO cev,
                                     CENS cen
                                where cev.VOT_CODVOT = vot.CODVOT and
                                      cen.CODCEN = cev.CEN_CODCEN and
                                     (cen.TIPCEN = 'GLB' or
                                      (cen.TIPCEN in ('EST', 'REG', 'LOC') and
                                       cen.EST_CODEST = P_CODEST)
                                     )
                                ) NUMCTDVOT,
                decode (vot.ESTVOT, 'FIN', to_char (vot.PRCPRT, 'fm990D00'),
                                           '-') PRCPRT,
                decode (vot.ESTVOT, 'FIN', (Select opv.CODOPV
                                              from OPCIO_VOTACIO opv
                                              where opv.VOT_CODVOT = vot.CODVOT and
                                                    opv.PRCVOT = (Select max(opv2.PRCVOT)
                                                                    from OPCIO_VOTACIO opv2
                                                                    where opv2.VOT_CODVOT = vot.CODVOT)),
                                           '-') CODOPVGUA,
                decode (vot.ESTVOT, 'FIN', (Select opv.PRCVOT
                                              from OPCIO_VOTACIO opv
                                              where opv.VOT_CODVOT = vot.CODVOT and
                                                    opv.PRCVOT = (Select max(opv2.PRCVOT)
                                                                    from OPCIO_VOTACIO opv2
                                                                    where opv2.VOT_CODVOT = vot.CODVOT)),
                                            '-') PRCOPVGUA
           from VOTACIO vot
           where exists (Select 1
                          from CENS_VOTACIO cev,
                               CENS cen
                          where cev.VOT_CODVOT = vot.CODVOT and
                                cen.CODCEN = cev.CEN_CODCEN and
                               (cen.TIPCEN = 'GLB' or
                                (cen.TIPCEN in ('EST', 'REG', 'LOC') and
                                 cen.EST_CODEST = P_CODEST)
                               )
                          )
           order by vot.DATINIVOT;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Donat un pas, obt el llistat de les votacions que s'hi han produt.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 900           Error inesperat
  --
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  -- %param P_LLISTAT OUT Cursor referenciat que cont les dades del llistat.
  PROCEDURE VOTACIONS_MES_DIFERENCIA (P_RSP       OUT VARCHAR2,
                                      P_LLISTAT   OUT SYS_REFCURSOR
                                     ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIONS_MES_DIFERENCIA', 'Obt la llista de votacions amb ms diferencia');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    If P_LLISTAT%ISOPEN then
      Close P_LLISTAT;
    end if;
      
    Open P_LLISTAT for
      Select rownum,
             x.*
        from (Select vot.INFTITVOT,
                     vot.DATINIVOT,
                     vot.DATFINVOT,
                     opvmes.CODOPV   COPOPVMAX,
                     opvmes.PRCVOT   PRCVOTMAX,
                     opvmenys.CODOPV COPOPVMIN,
                     opvmenys.PRCVOT PRCVOTMIN,
                     (opvmes.PRCVOT - opvmenys.PRCVOT) DIFPRCVOT
                from VOTACIO vot,
                     OPCIO_VOTACIO opvmes,
                     OPCIO_VOTACIO opvmenys
                where vot.ESTVOT = 'FIN' and
                      opvmes.VOT_CODVOT = vot.CODVOT and
                      opvmes.CODOPV = VOTACIO_PKG.OPCIO_AMB_MES_VOTS (vot.CODVOT) and
                      opvmenys.VOT_CODVOT = vot.CODVOT and
                      opvmenys.CODOPV = VOTACIO_PKG.OPCIO_AMB_MENYS_VOTS (vot.CODVOT)
              order by 8 desc) x
        where rownum <= 10;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Obt el llistat de les 10 votacions que han tingut menys diferencia entre la
  -- opci ms votada i la menys votada
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 900           Error inesperat
  --
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  -- %param P_LLISTAT OUT Cursor referenciat que cont les dades del llistat.
  PROCEDURE VOTACIONS_MENYS_DIFERENCIA (P_RSP       OUT VARCHAR2,
                                        P_LLISTAT   OUT SYS_REFCURSOR
                                       ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIONS_MENYS_DIFERENCIA', 'Obt la llista de votacions amb menys diferencia');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    If P_LLISTAT%ISOPEN then
      Close P_LLISTAT;
    end if;
      
    Open P_LLISTAT for
      Select rownum,
             x.*
        from (Select vot.INFTITVOT,
                     vot.DATINIVOT,
                     vot.DATFINVOT,
                     opvmes.CODOPV   COPOPVMAX,
                     opvmes.PRCVOT   PRCVOTMAX,
                     opvmenys.CODOPV COPOPVMIN,
                     opvmenys.PRCVOT PRCVOTMIN,
                     (opvmes.PRCVOT - opvmenys.PRCVOT) DIFPRCVOT
                from VOTACIO vot,
                     OPCIO_VOTACIO opvmes,
                     OPCIO_VOTACIO opvmenys
                where vot.ESTVOT = 'FIN' and
                      opvmes.VOT_CODVOT = vot.CODVOT and
                      opvmes.CODOPV = VOTACIO_PKG.OPCIO_AMB_MES_VOTS (vot.CODVOT) and
                      opvmenys.VOT_CODVOT = vot.CODVOT and
                      opvmenys.CODOPV = VOTACIO_PKG.OPCIO_AMB_MENYS_VOTS (vot.CODVOT)
              order by 8 asc) x
        where rownum <= 10;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Donat el codi d'un ciutad, obt els censos als quals pertany.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi del ciutad
  -- 101           El ciutad no existeix
  -- 900           Error inesperat
  --
  -- %param P_CODESt      Codi d'estat
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  -- %param P_LLISTAT OUT Cursor referenciat que cont les dades del llistat.
  PROCEDURE CENSOS_CIUTADA (P_CODCTD        CIUTADA.CODCTD%TYPE,
                            P_RSP       OUT VARCHAR2,
                            P_LLISTAT   OUT SYS_REFCURSOR
                            ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CENSOS_CIUTADA', 'Obt la llista de censos als que pertany un ciutad');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CODI de ciutad s obligatori';
    elsif not CIUTADA_PKG.CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '101-No existeix cap ciutad amb el CODI ' || P_CODCTD;
    end if;

    -- s tot correcte, realitzem el llistat
    If P_RSP = 'OK' then

      If P_LLISTAT%ISOPEN then
        Close P_LLISTAT;
      end if;
      
      Open P_LLISTAT for
        Select cen.CODCEN
          from CENS cen
          connect by prior cen.CEN_CODCEN = cen.CODCEN
          start with (cen.EST_CODEST, cen.REG_CODREG, cen.LOC_CODLOC) in  (Select ctd.EST_CODEST, ctd.REG_CODREG, ctd.LOC_CODLOC
                                                                              from CIUTADA ctd
                                                                              where ctd.CODCTD = P_CODCTD);
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;
  
  -- Donat el codi d'un ciutad, obt les votacions en que ha participat i les que no
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi del ciutad
  -- 101           El ciutad no existeix
  -- 900           Error inesperat
  --
  -- %param P_CODESt      Codi d'estat
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  -- %param P_LLISTAT OUT Cursor referenciat que cont les dades del llistat.
  PROCEDURE VOTACIONS_CIUTADA (P_CODCTD        CIUTADA.CODCTD%TYPE,
                               P_RSP       OUT VARCHAR2,
                               P_LLISTAT   OUT SYS_REFCURSOR
                              ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIONS_CIUTADA', 'Obt la llista de votacions en les que ha participat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CODI de ciutad s obligatori';
    elsif not CIUTADA_PKG.CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '101-No existeix cap ciutad amb el CODI ' || P_CODCTD;
    end if;

    -- s tot correcte, realitzem el llistat
    If P_RSP = 'OK' then

      If P_LLISTAT%ISOPEN then
        Close P_LLISTAT;
      end if;
      
      Open P_LLISTAT for
        Select vot.INFTITVOT,
               vot.DATINIVOT,
               vot.DATFINVOT,
               decode ((Select count(1)
                         from dual
                         where exists (Select 1
                                         from VOT_EMES voe
                                         where voe.VOT_CODVOT = vot.CODVOT and
                                               voe.CTD_CODCTD = P_CODCTD)), 0, 'NO',
                                                                            1, 'S') FLGPRT
          from VOTACIO vot
          where VOTACIO_PKG.CIUTADA_AMB_DRET_A_VOT (P_CODCTD, vot.CODVOT) = 1;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;
  
end;
/

exit
/

