create or replace package body ESTADISTIQUES_PKG is

  -- Obtenci del percentatge de participaci en una votaci finalitzada
  --
  -- %param P_CODVOT  Codi de Votaci
  --
  -- %return Percentatge de participaci
  FUNCTION OBT_PARTICIPACIO_VOTACIO (P_CODVOT           VOTACIO.CODVOT%TYPE
                                    ) RETURN VOTACIO.PRCPRT%TYPE IS

    Cursor cVot is
      Select vot.PRCPRT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT;

    vPrcPrt    VOTACIO.PRCPRT%TYPE;
  BEGIN
    Open cVot;
    Fetch cVot into vPrcPrt;
    Close cVot;
    
    return vPrcPrt;
  END;

  -- Calcula la diferncia percentual entre la opci ms votada i
  -- la menys votada d'una votaci.
  --
  -- %param P_CODVOT  Codi votaci
  --
  -- %return Percentatge de la diferncia percentual entre la opci
  --         ms votada i la menys
  FUNCTION OBT_DIFERENCIA_OPCIONS (P_CODVOT    VOTACIO.CODVOT%TYPE
                                  ) RETURN NUMBER IS
    vDifVot    number;
  BEGIN
    Select vot.NUMDIFOPV
      into vDifVot
      from VOTACIO vot
      where vot.CODVOT = P_CODVOT;

    return vDifVot;
  END;

  -- Obt el percentatge de ciutadans que no han exercit el seu dret a vot
  -- en un any concret.
  --
  -- El percentatge es calcula sobre el total de ciutadans que a 31 de desembre
  -- son major d'edat.
  --
  -- Nota: es consideren votacions finalitzades en l'any indicat
  --
  -- %param P_NUMANY  Any a comprovar
  --
  -- %return Percentatge de ciutadans que no han exercit el seu dret a vot.
  FUNCTION OBT_PRC_CTD_NO_VOT_ANY (P_NUMANY    NUMBER
                                  ) RETURN NUMBER IS

    vNumTotCtd    number;
    vNumCtdNvo    number;
    vPrcNvo       number;
  BEGIN
    -- Nmero total de ciutadans amb dret a vot durant l'any
    Select count(1)
      into vNumTotCtd
      from CIUTADA ctd
      where ((to_date ('31-12-' || P_NUMANY, 'dd-mm-rrrr') - trunc (ctd.DATNAC)) / 365) >= 18;

    -- Nmero de ciutadans amb dret a vot durant l'any que no han votat en
    -- cap votaci finalitzada durant el mateix
    Select count(1)
      into vNumCtdNvo
      from CIUTADA ctd
      where ((to_date ('31-12-' || P_NUMANY, 'dd-mm-rrrr') - trunc (ctd.DATNAC)) / 365) >= 18 and
            not exists (Select 1
                          from VOTACIO vot,
                               VOT_EMES voe
                          where vot.CODVOT = voe.VOT_CODVOT and
                                extract (year from vot.DATFINVOT) = P_NUMANY and
                                voe.CTD_CODCTD = ctd.CODCTD and
                                vot.ESTVOT = 'FIN');

    vPrcNvo := round ((vNumCtdNvo / vNumTotCtd) * 100, 2);

    return vPrcNvo;
  END;

  -- Extreu les estadstiques de la votaci indicada
  --
  -- %param P_rVot  Registre amb totes les dades d'una fila de la taula votaci.
  PROCEDURE ESTADISTIQUES_VOTACIO (P_rVOT    VOTACIO%ROWTYPE
                                  ) IS


    Cursor cEsv is
      Select esv.*
        from ESD_VOTACIO esv;

    -- Ciutadans que han ems el seu vot
    Cursor cVem is
      Select vem.CTD_CODCTD, ctd.EST_CODEST
        from VOT_EMES vem,
             CIUTADA ctd
        where vem.VOT_CODVOT = P_rVot.CODVOT and
              ctd.CODCTD = vem.CTD_CODCTD;

    vCodCtd       CIUTADA.CODCTD%TYPE;

    -- Participacions d'un ciutad
    Cursor cEct is
      Select ect.NUMPRTVOT
        from ESD_CIUTADA ect
        where ect.CTD_CODCTD = vCodCtd
          for update nowait;
    
    rEsv          ESD_VOTACIO%ROWTYPE;
    vMaxDif       number;
    vMinDif       number;
    vMaxPrtCtd    number;
    vMaxEstPrt    ESTAT.CODEST%TYPE;
    vNumVot       number;
    vCodEst       ESTAT.CODEST%TYPE;
  BEGIN  
    -- Actualitzar dades de votacions per ciutad
    vMaxPrtCtd := 0;
    vMaxEstPrt := null;
      
    Open cVem;
    Fetch cVem into vCodCtd, vCodEst;
    While cVem%FOUND loop
    
      vNumVot := 0;
        
      Open cEct;
      Fetch cEct into vNumVot;
        
      If cEct%FOUND then
        Update ESD_CIUTADA ect
          set ect.NUMPRTVOT = ect.NUMPRTVOT + 1
          where current of cEct;
      else
        Insert into ESD_CIUTADA (CTD_CODCTD, NUMPRTVOT)
          values (vCodCtd, 1);
      end if;
        
      Close cEct;
      
      If (vNumVot + 1) > vMaxPrtCtd then
        vMaxPrtCtd := vNumVot + 1;
        vMaxEstPrt := vCodEst;
      end if;
      
      Fetch cVem into vCodCtd, vCodEst;
    end loop;
    Close cVem;
      
    -- Llegim estadstiques de votaci actual.
    -- Noms hi pot haver 1 registre.
    -- En cas de no trovar-lo, vol dir que etem tractant la primera
    -- votaci del sistema
    Open cEsv;
    Fetch cEsv into rEsv;
    If cEsv%NOTFOUND then
      -- Inserim un nou registre prenent com a valors vlids els de
      -- la votaci indicada.
      rEsv.VOT_CODVOT_MAXPRT    := P_rVot.CODVOT;
      rEsv.VOT_CODVOT_MAXDIF    := P_rVot.CODVOT;
      rEsv.VOT_CODVOT_MINDIF    := P_rVot.CODVOT;
      rEsv.NUMMAXPRTCTD         := vMaxPrtCtd;
      rEsv.EST_CODEST_MAXPRTCTD := vMaxEstPrt;
      
      Insert into ESD_VOTACIO
        values rEsv;
    else

      -- Mxima participaci
      If P_rVot.PRCPRT > OBT_PARTICIPACIO_VOTACIO (rEsv.VOT_CODVOT_MAXPRT) then
        rEsv.VOT_CODVOT_MAXPRT := P_rVot.CODVOT;
      end if;

      -- Mxima diferncia
      vMaxDif := OBT_DIFERENCIA_OPCIONS (rEsv.VOT_CODVOT_MAXDIF);
      If OBT_DIFERENCIA_OPCIONS (P_rVot.CODVOT) > vMaxDif then
        rEsv.VOT_CODVOT_MAXDIF := P_rVot.CODVOT;
      end if;

      -- Mnima diferncia
      vMinDif := OBT_DIFERENCIA_OPCIONS (rEsv.VOT_CODVOT_MINDIF);
      If OBT_DIFERENCIA_OPCIONS (P_rVot.CODVOT) < vMinDif then
        rEsv.VOT_CODVOT_MINDIF := P_rVot.CODVOT;
      end if;

      -- Mxima participaci d'un ciutad
      If vMaxPrtCtd > rEsv.NUMMAXPRTCTD then
        rEsv.NUMMAXPRTCTD := vMaxPrtCtd;
        rEsv.EST_CODEST_MAXPRTCTD := vMaxEstPrt;
      end if;
      
      Update ESD_VOTACIO
        set row = rEsv;
    end if;
    Close cEsv;
  END;

  -- Extreu les estadstiques anulas de la votaci i any indicats
  --
  -- %param P_NUMANY  Any a estudiar
  -- %param P_rVot    Registre amb totes les dades d'una fila de la taula votaci.
  PROCEDURE ESTADISTIQUES_ANY (P_NUMANY    ESD_ANY.NUMANY%TYPE,
                               P_rVOT      VOTACIO%ROWTYPE
                              ) IS


    Cursor cEsa is
      Select esa.*
        from ESD_ANY esa
        where esa.NUMANY = P_NUMANY;

    Cursor cLoc is
      Select ctd.EST_CODEST, ctd.REG_CODREG, ctd.LOC_CODLOC
        from VOT_EMES vem,
             CIUTADA ctd
        where vem.CTD_CODCTD = ctd.CODCTD and
              vem.VOT_CODVOT = P_rVot.CODVOT;

    rEsa          ESD_ANY%ROWTYPE;
    rLoc          cLoc%ROWTYPE;
    
    Cursor cEla is
      Select ela.NUMVOT
        from ESD_LOCALITAT_ANY ela
        where ela.NUMANY = P_NUMANY and
              ela.EST_CODEST = rLoc.EST_CODEST and
              ela.REG_CODREG = rLoc.REG_CODREG and
              ela.LOC_CODLOC = rLoc.LOC_CODLOC
          for update nowait;
    
    vNumVot       number;
    vPrcPrt       VOTACIO.PRCPRT%TYPE;
    vMaxCodEst    LOCALITAT.EST_CODEST%TYPE;
    vMaxCodReg    LOCALITAT.REG_CODREG%TYPE;
    vMaxCodLoc    LOCALITAT.CODLOC%TYPE;
    vNumMaxVot    number;
  BEGIN
    -- Actualitzem les estadstiques de votacins per localitat
    vNumMaxVot := 0;
    
    Open cLoc;
    Fetch cLoc into rLoc;
    While cLoc%FOUND loop
    
      -- Actualitzem el nmero de vots de la localitat
      Open cEla;
      Fetch cEla into vNumVot;
      
      If cEla%FOUND then
        Update ESD_LOCALITAT_ANY ela
          set ela.NUMVOT = ela.NUMVOT + 1
          where current of cEla;
      else
        Insert into ESD_LOCALITAT_ANY
          values (P_NUMANY, rLoc.EST_CODEST, rLoc.REG_CODREG, rLoc.LOC_CODLOC, 1);
      end if;
      
      Close cEla;
      
      If (vNumvot + 1) > vNumMaxVot then
        vNumMaxVot := vNumVot + 1;
        vMaxCodEst := rLoc.EST_CODEST;
        vMaxCodReg := rLoc.REG_CODREG;
        vMaxCodLoc := rLoc.LOC_CODLOC;
      end if;
    
      Fetch cLoc into rLoc;
    end loop;
    Close cLoc;
    
    -- Llegim estadstiques de l'any indicat
    -- Noms hi pot haver 1 registre.
    -- En cas de no trovar-lo, vol dir que etem tractant la primera
    -- votaci del sistema
    Open cEsa;
    Fetch cEsa into rEsa;
    If cEsa%NOTFOUND then

      -- Inserim un nou registre prenent com a valors vlids els de
      -- la votaci indicada.
      rEsa.NUMANY     := P_NUMANY;
      rEsa.VOT_CODVOT := P_rVot.CODVOT;
      rEsa.EST_CODEST := vMaxCodEst;
      rEsa.REG_CODREG := vMaxCodReg;
      rEsa.LOC_CODLOC := vMaxCodLoc;
      rEsa.NUMMAXVOT  := vNumMaxVot;
      rEsa.PRCCTDNOV  := OBT_PRC_CTD_NO_VOT_ANY (P_NUMANY);

      Insert into ESD_ANY
        values rEsa;
    else
      -- Votaci amb ms participaci
      vPrcPrt := OBT_PARTICIPACIO_VOTACIO (rEsa.VOT_CODVOT);
      If P_rVot.PRCPRT > vPrcPrt then
        rEsa.VOT_CODVOT := P_rVot.CODVOT;
      end if;

      -- Localitat amb ms vots emesos durant l'any
      If vNumMaxVot > rEsa.NUMMAXVOT then
        rEsa.EST_CODEST := vMaxCodEst;
        rEsa.REG_CODREG := vMaxCodReg;
        rEsa.LOC_CODLOC := vMaxCodLoc;
        rEsa.NUMMAXVOT  := vNumMaxVot;
      end if;

      -- Percentatge de ciutadans que no han votat mai durant l'any
      rEsa.PRCCTDNOV := OBT_PRC_CTD_NO_VOT_ANY (P_NUMANY);

      Update ESD_ANY esa
        set row = rEsa
        where esa.NUMANY = P_NUMANY;
    end if;
    Close cEsa;
  END;

  -- Extreu les estadstiques de l'estat i any indicats
  --
  -- %param P_NUMANY  Any a estudiar
  -- %param P_CODEST  Estat a estudiar
  PROCEDURE ESTADISTIQUES_ESTAT_ANY (P_NUMANY    ESD_ESTAT_ANY.NUMANY%TYPE,
                                     P_CODEST    ESD_ESTAT_ANY.EST_CODEST%TYPE
                                    ) IS


    Cursor cEea is
      Select eea.*
        from ESD_ESTAT_ANY eea
        where eea.NUMANY = P_NUMANY and
              eea.EST_CODEST = P_CODEST;

    rEea         ESD_ESTAT_ANY%ROWTYPE;
  BEGIN
    -- Llegim estadstiques de l'any indicat
    -- Noms hi pot haver 1 registre.
    -- En cas de no trovar-lo, vol dir que estem tractant la primera
    -- votaci del sistema
    Open cEea;
    Fetch cEea into rEea;
    If cEea%NOTFOUND then

      -- Inserim un nou registre prenent com a valors vlids els de
      -- la votaci indicada.
      rEea.NUMANY     := P_NUMANY;
      rEea.EST_CODEST := P_CODEST;
      rEea.NUMVOTFIN  := 1;
      
      Insert into ESD_ESTAT_ANY eea
        values rEea;
    else
      Update ESD_ESTAT_ANY eea
        set eea.NUMVOTFIN = eea.NUMVOTFIN + 1
        where eea.NUMANY = P_NUMANY and
              eea.EST_CODEST = P_CODEST;
    end if;
    Close cEea;
  END;

  -- Extreu les estadstiques per cens i any indicats
  --
  -- %param P_NUMANY  Any a estudiar
  -- %param P_CODCEN  cens a estudiar
  PROCEDURE ESTADISTIQUES_CENS_ANY (P_NUMANY    ESD_CENS_ANY.NUMANY%TYPE,
                                    P_CODCEN    ESD_CENS_ANY.CEN_CODCEN%TYPE
                                   ) IS


    Cursor cEca is
      Select eca.*
        from ESD_CENS_ANY eca
        where eca.NUMANY = P_NUMANY and
              eca.CEN_CODCEN = P_CODCEN;

    rEca       ESD_CENS_ANY%ROWTYPE;
    vNumCtd    number;
    vPrcPrt    number;
    vNumVot    number;
    vPrcPrtTot number;
    vNumTotVot number;
  BEGIN

    -- Llegim estadstiques de l'any indicat
    -- Noms hi pot haver 1 registre.
    -- En cas de no trovar-lo, vol dir que etem tractant la primera
    -- votaci del sistema
    Open cEca;
    Fetch cEca into rEca;

    rEca.NUMANY     := P_NUMANY;
    rEca.CEN_CODCEN := P_CODCEN;


    vPrcPrtTot := 0;
    vNumTotVot := 0;

    For rVot in (Select vot.CODVOT, vot.DATFINVOT
                   from VOTACIO vot
                   where extract (year from vot.DATFINVOT) = P_NUMANY and
                         vot.ESTVOT = VOTACIO_PKG.cFINALITZADA and
                         exists (Select 1
                                   from CENS_VOTACIO cev
                                   where cev.VOT_CODVOT = vot.CODVOT and
                                         cev.CEN_CODCEN = P_CODCEN)) loop

      vNumCtd := CENS_PKG.NOMBRE_CIUTADANS_CENS (P_CODCEN, rVot.DATFINVOT);

      Select count(1)
        into vNumVot
        from VOT_EMES voe
        where voe.VOT_CODVOT = rVot.CODVOT and
              CENS_PKG.CIUTADA_PERTANY_A_CENS (voe.CTD_CODCTD, P_CODCEN) = 1;

      vPrcPrt := round ((vNumVot / vNumCtd) * 100, 2);
      vPrcPrtTot := vPrcPrtTot + vPrcPrt;
      vNumTotVot := vNumTotVot + 1;
    end loop;

    rEca.PRCPRTMIG := round ((vPrcPrtTot / vNumTotVot), 2);
    rEca.NUMVOTCEN := vNumTotVot;

    If cEca%NOTFOUND then
      Insert into ESD_CENS_ANY eca
        values rEca;
    else
      Update ESD_CENS_ANY eca
        set row = rEca
        where eca.NUMANY = P_NUMANY and
              eca.CEN_CODCEN = P_CODCEN;
    end if;
    Close cEca;
  END;

  -- Procs de tancament duna votaci. Realitza el cmput de participaci i llana
  -- el procs dextracci de les dades estadstiques. Noms es pot finalitzar votacions
  -- que estan en estat oberta i que la data de sistema es superior a la data de fi de
  -- votaci.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar el codi de la votaci
  -- 101           La votaci no existeix
  -- 102           La votaci no est en un estat que permeti realitzar el tancament.
  -- 103           El perode de votaci no ha finalitzat.
  -- 104           Error en el procs destadstiques.
  -- 900           Error inesperat
  --
  -- %param P_CODVOT      Codi del cens
  -- %param P_RSP    OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                      valor OK, sin retorna un codi derror  seguit de la descripci,
  --                      seguint el format CODI-DESCRIPCIӔ
  PROCEDURE EXTREURE_DADES_VOTACIO (P_CODVOT        VOTACIO.CODVOT%TYPE,
                                    P_RSP       OUT VARCHAR2
                                   ) IS

    Cursor cVot is
      Select vot.*
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT;

    vIdeLog      LOG_INFO.IDELOG%TYPE;
    rVot         VOTACIO%ROWTYPE;
    vNumAny      number;
    vExisteix    number;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('EXTREURE_DADES_VOTACIO', 'Extreure dades d''una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP        := 'OK';

    -- Llegim dades de la votaci
    Open cVot;
    Fetch cVot into rVot;
    Close cVot;

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-La VOTACI s obligatoria.';
    elsif not VOTACIO_PKG.VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '101-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif rVot.ESTVOT != VOTACIO_PKG.cFINALITZADA then
      P_RSP := '102-La votaci no est tancada.';
    end if;

    -- s tot correcte, comprovem si existeix el ciutad
    If P_RSP = 'OK' then

      vNumAny := extract (year from rVot.DATFINVOT);

      -- Dades estadstiques per votaci
      ESTADISTIQUES_VOTACIO (rVot);

      -- Dades estadstiques per any
      ESTADISTIQUES_ANY (vNumAny, rVot);

      -- Dades estadstiques per cens
      For rCen in (Select cev.CEN_CODCEN
                     from CENS_VOTACIO cev
                     where cev.VOT_CODVOT = P_CODVOT) loop
        ESTADISTIQUES_CENS_ANY (vNumAny, rCen.CEN_CODCEN);
      end loop;

      -- Dades estadstiques per estat i any
      -- Si la votaci t assignat el cens global, llavors considerarem
      -- que tots els estats que estiguin donats d'alta han participat en
      -- la votaci
      Select count(1)
        into vExisteix
        from dual
        where exists (Select 1
                        from CENS_VOTACIO cev
                        where cev.VOT_CODVOT = P_CODVOT and
                              cev.CEN_CODCEN = CENS_PKG.OBTENIR_CENS_GLOBAL);

      -- Tots els estats
      If vExisteix = 1 then
        For rEst in (Select est.CODEST
                       from ESTAT est
                    ) loop
          ESTADISTIQUES_ESTAT_ANY (vNumAny, rEst.CODEST);
        end loop;

      -- Estats assignats a la votaci
      else
        -- Si la votaci t assignat el cens global, llavors considerarem tots
        -- els estats del sistema
        Select count(1)
          into vExisteix
          from dual
          where exists (Select 1
                          from CENS_VOTACIO cev,
                               CENS cen
                         where cen.CODCEN = cev.CEN_CODCEN and
                               cev.VOT_CODVOT = P_CODVOT and
                               cen.TIPCEN = CENS_PKG.cGLOBAL);
        If vExisteix = 1 then
          For rEst in (Select est.CODEST
                         from ESTAT est) loop
            ESTADISTIQUES_ESTAT_ANY (vNumAny, rEst.CODEST);
          end loop;
        else             
          For rEst in (Select distinct cen.EST_CODEST
                         from CENS_VOTACIO cev,
                              CENS cen
                         where cen.CODCEN = cev.CEN_CODCEN and
                               cev.VOT_CODVOT = P_CODVOT and
                               cen.EST_CODEST is not null) loop
            ESTADISTIQUES_ESTAT_ANY (vNumAny, rEst.EST_CODEST);
          end loop;
        end if;
      end if;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

end;
/

exit
/

