create or replace package GEOGRAFIC_PKG is

  -- Comprova l'existncia d'un estat.
  --
  -- %param P_CODEST  Estat a comprovar
  --
  -- %return Indicador sobre si existeix un estat amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION ESTAT_EXISTEIX (P_CODEST    ESTAT.CODEST%TYPE
                          ) RETURN BOOLEAN;

  -- Comprova si l'estat indicat t dades asociades
  -- Dades a comprovar:
  --  Regions
  --  Ciutadans
  --
  -- %param P_CODEST  Estat a comprovar
  --
  -- %return Indicador sobre si l'estat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION ESTAT_TE_DADES_ASOCIADES (P_CODEST    ESTAT.CODEST%TYPE
                                    ) RETURN BOOLEAN;

  -- Dona d'alta un nou estat.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar el nom de lestat
  -- 102            Lestat ja existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_NOMEST  Nom de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_ALTA (P_CODEST        ESTAT.CODEST%TYPE,
                        P_NOMEST        ESTAT.NOMEST%TYPE,
                        P_RSP       OUT VARCHAR2);


  -- Modifica les dades d'un estat (el nom)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar el nom de lestat
  -- 102            Lestat no existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_NOMEST  Nou nom de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_MODIFICACIO (P_CODEST        ESTAT.CODEST%TYPE,
                               P_NOMEST        ESTAT.NOMEST%TYPE,
                               P_RSP       OUT VARCHAR2);

  -- Dona de baixa un estat. Noms es permet donar de baixa un estat si aquest
  -- t dades associades (regions, censos...)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Lestat no existeix
  -- 102            Lestat t dades associades. No es permet donar-lo de baixa
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_BAIXA (P_CODEST        ESTAT.CODEST%TYPE,
                         P_RSP       OUT VARCHAR2);

  -- Comprova l'existncia d'una regi.
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  --
  -- %return Indicador sobre si existeix una regi amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION REGIO_EXISTEIX (P_CODEST    REGIO.EST_CODEST%TYPE,
                           P_CODREG    REGIO.CODREG%TYPE
                          ) RETURN BOOLEAN;

  -- Comprova si l'estat indicat t dades asociades
  -- Dades a comprovar:
  --  Localitats
  --  Ciutadans
  --  Censos
  --
  -- %param P_CODEST  Estat a comprovar
  --
  -- %return Indicador sobre si l'estat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION REGIO_TE_DADES_ASOCIADES (P_CODEST    REGIO.EST_CODEST%TYPE,
                                     P_CODREG    REGIO.CODREG%TYPE
                                    ) RETURN BOOLEAN;

  -- Dona dalta una regi.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar el nom de la regi
  -- 103            Lestat no existeix
  -- 104            La regi ja existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_ALTA (P_CODEST        REGIO.EST_CODEST%TYPE,
                        P_CODREG        REGIO.CODREG%TYPE,
                        P_NOMREG        REGIO.NOMREG%TYPE,
                        P_RSP       OUT VARCHAR2);

  -- Modifica les dades duna regio (nom).
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar el nom de la regi
  -- 103            Lestat no existeix
  -- 104            La regi no existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_MODIFICACIO (P_CODEST        REGIO.EST_CODEST%TYPE,
                               P_CODREG        REGIO.CODREG%TYPE,
                               P_NOMREG        REGIO.NOMREG%TYPE,
                               P_RSP       OUT VARCHAR2);

  -- Dona de baixa una regi. Noms es permet donar de baixa una regi si
  -- aquesta no t dades associades (localitats, censos...)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Lestat no existeix
  -- 103            La regi no existeix
  -- 104            La regi t dades associades i no es permet la seva baixa.
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_BAIXA (P_CODEST        REGIO.EST_CODEST%TYPE,
                         P_CODREG        REGIO.CODREG%TYPE,
                         P_RSP       OUT VARCHAR2);


  -- Comprova l'existncia d'una localitat.
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  -- %param P_CODLOC  Localitat a comprovar
  --
  -- %return Indicador sobre si existeix una localitat amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION LOCALITAT_EXISTEIX (P_CODEST    LOCALITAT.EST_CODEST%TYPE,
                               P_CODREG    LOCALITAT.REG_CODREG%TYPE,
                               P_CODLOC    LOCALITAT.CODLOC%TYPE
                              ) RETURN BOOLEAN;

  -- Comprova si la localitat indicada t dades asociades
  -- Dades a comprovar:
  --  Ciutadans
  --  Censos
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  -- %param P_CODLOC  Localitat a comprovar
  --
  -- %return Indicador sobre si la localitat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION LOCALITAT_TE_DADES_ASOCIADES (P_CODEST    LOCALITAT.EST_CODEST%TYPE,
                                         P_CODREG    LOCALITAT.REG_CODREG%TYPE,
                                         P_CODLOC    LOCALITAT.CODLOC%TYPE
                                        ) RETURN BOOLEAN;

  -- Dona dalta una localitat.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Sha dinformar el nom de la regi
  -- 104            Lestat no existeix
  -- 105            La regi no existeix
  -- 106            La localitat ja existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_ALTA (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                            P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                            P_CODLOC        LOCALITAT.CODLOC%TYPE,
                            P_NOMLOC        LOCALITAT.NOMLOC%TYPE,
                            P_RSP       OUT VARCHAR2);

  -- Dona dalta una localitat.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Sha dinformar el nom de la regi
  -- 104            Lestat no existeix
  -- 105            La regi no existeix
  -- 106            La localitat no existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_MODIFICACIO (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                                   P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                                   P_CODLOC        LOCALITAT.CODLOC%TYPE,
                                   P_NOMLOC        LOCALITAT.NOMLOC%TYPE,
                                   P_RSP       OUT VARCHAR2);

  -- Dona de baixa una localitat. Noms es pot donar de baixa si no t
  -- dades associades
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Lestat no existeix
  -- 104            La regi no existeix
  -- 105            La localitat no existeix
  -- 106            La localitat t dades asociades. No es permet la seva baixa
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_BAIXA (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                             P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                             P_CODLOC        LOCALITAT.CODLOC%TYPE,
                             P_RSP       OUT VARCHAR2);
end;
/

exit
/

