create or replace package body LOG_PKG is

  -- Descriu l'estat de procs indicat
  --
  -- %param P_LSTESTPRC  Estat
  --
  -- %return Descripci de l'estat
  FUNCTION DESC_ESTAT_PROCES (P_LSTESTPRC    LOG_INFO.LSTESTPRC%TYPE
                             ) RETURN VARCHAR2 IS

    vDescEstat    varchar2(20);
  BEGIN
    Case P_LSTESTPRC
      when cESTAT_INICIALITZAT then
        vDescEstat := 'Inicialitzat';
      when cESTAT_FINALITZAT then
        vDescEstat := 'Finalitzat';
      when cESTAT_AMB_ERRORS then
        vDescEstat := 'Finalitzat amb errors';
      else
        vDescEstat := P_LSTESTPRC;
    end case;

    return vDescEstat;
  END;

  -- Inicialitza un log de procs.
  -- NOTA: usa una transacci autnoma i realitzar COMMIT.
  --
  -- %param P_INFPRC  Nom del proces logat
  -- %param P_INFOBS  Observacions
  --
  -- %return Identificador del log iniciat
  FUNCTION INICIAR_LOG (P_INFPRC    LOG_INFO.INFPRC%TYPE,
                        P_INFOBS    LOG_INFO.INFOBS%TYPE
                       ) RETURN LOG_INFO.IDELOG%TYPE IS

    PRAGMA AUTONOMOUS_TRANSACTION;

    rLog    LOG_INFO%ROWTYPE;
  BEGIN

    Select SEQ_LOG_INFO.nextval into rLog.IDELOG from dual;

    rLog.INFPRC    := P_INFPRC;
    rLog.INFOBS    := P_INFOBS;
    rLog.FECINIPRC := sysdate;
    rLog.LSTESTPRC := cESTAT_INICIALITZAT;

    Insert into LOG_INFO
      values rLog;

    commit;

    return rLog.IDELOG;
  END;

  -- Registra una lnia de detalle del log d'un procs
  -- NOTA: usa una transacci autnoma i realitzar COMMIT.
  --
  -- %param P_IDELOG  Id. del log on es vol registrar el detall
  -- %param P_INFDLG  Text del detall de log
  PROCEDURE REGISTRAR_DETALL_LOG (P_IDELOG    DETALL_LOG.LOG_IDELOG%TYPE,
                                  P_INFDLG    DETALL_LOG.INFDLG%TYPE
                                 ) IS

    PRAGMA AUTONOMOUS_TRANSACTION;

    rDlg    DETALL_LOG%ROWTYPE;
  BEGIN

    -- Segent nmero de detall
    Select nvl (max (dlg.NUMDLG), 0) + 1
      into rDlg.NUMDLG
      from DETALL_LOG dlg
      where dlg.LOG_IDELOG = P_IDELOG;

    rDlg.LOG_IDELOG := P_IDELOG;
    rDlg.INFDLG     := P_INFDLG;

    Insert into DETALL_LOG
      values rDlg;

    commit;
  END;

  -- Finalitza un log de procs. Es guarda com a data de fi la
  -- data de sisteam
  -- NOTA: usa una transacci autnoma i realitzar COMMIT.
  --
  -- %param P_IDELOG     Id. de log a finalitzar
  -- %param P_LSTESTPRC  Esta final del procs
  --                         {*} FIN  Finalitzat correctament
  --                         {*} ERR  Finalitzat amb errors
  PROCEDURE FINALITZAR_LOG (P_IDELOG       LOG_INFO.IDELOG%TYPE,
                            P_LSTESTPRC    LOG_INFO.LSTESTPRC%TYPE
                           ) IS

    PRAGMA AUTONOMOUS_TRANSACTION;

  BEGIN
    -- ACtualitzem la data de fi del procs i l'estat
    Update LOG_INFO log
      set log.FECFINPRC = sysdate,
          log.LSTESTPRC = P_LSTESTPRC
      where log.IDELOG = P_IDELOG;

    commit;
  END;

  -- Mostra la informaci de log a travs de l'"OUTPUT"
  --
  -- %param P_IDELOG  Id del log a mostrar
  PROCEDURE MOSTRAR_LOG (P_IDELOG    LOG_INFO.IDELOG%TYPE) IS

    rLog    LOG_INFO%ROWTYPE;
  BEGIN
    -- Obtenim les dades de log
    Select log.*
      into rLog
      from LOG_INFO log
      where log.IDELOG = P_IDELOG;

    -- Mostrem les dades
    dbms_output.put_line ('Informaci procs');
    dbms_output.put_line ('==================================================');
    dbms_output.put_line ('Procs:       ' || rLog.INFPRC);
    dbms_output.put_line ('Estat:        ' || DESC_ESTAT_PROCES (rLog.LSTESTPRC));
    dbms_output.put_line ('Inici:        ' || to_char (rLog.FECINIPRC, 'dd-mm-rrrr hh24:mi:ss'));
    dbms_output.put_line ('Fi:           ' || to_char (rLog.FECFINPRC, 'dd-mm-rrrr hh24:mi:ss'));

    If rLog.INFOBS is not null then
      dbms_output.put_line ('Observacions: ' || rLog.INFOBS);
    end if;

    dbms_output.put_line (' ');
    dbms_output.put_line ('Detall');
    dbms_output.put_line ('==================================================');
    dbms_output.put_line (' ');

    -- Mostrem el detalle de log
    For rDlg in (Select dlg.INFDLG
                   from DETALL_LOG dlg
                   where dlg.LOG_IDELOG = P_IDELOG
                   order by dlg.NUMDLG asc) loop

      dbms_output.put_line (rDlg.INFDLG);
    end loop;
  END;

end;
/

exit
/

