create or replace package body VOTACIO_PKG is

  -- Esborra totes les opcions d'una votaci.
  --
  -- %param P_CODVOT Codi de votaci
  PROCEDURE ESBORRAR_OPCIONS_VOTACIO (P_CODVOT    VOTACIO.CODVOT%TYPE
                                     ) IS

    Cursor cOpv is
      Select 1
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT
        for update nowait;
  BEGIN
    -- Bloquejem els registres
    Open cOpv;
    Close cOpv;

    -- Esborrem
    Delete from OPCIO_VOTACIO opv
      where opv.VOT_CODVOT = P_CODVOT;
  END;

  -- Esborra tots els censos assignats a una votaci.
  --
  -- %param P_CODVOT Codi de votaci
  PROCEDURE ESBORRAR_CENSOS_VOTACIO (P_CODVOT    VOTACIO.CODVOT%TYPE
                                    ) IS

    Cursor cCev is
      Select 1
        from CENS_VOTACIO cev
        where cev.VOT_CODVOT = P_CODVOT
        for update nowait;
  BEGIN
    -- Bloquejem els registres
    Open cCev;
    Close cCev;

    -- Esborrem
    Delete from CENS_VOTACIO cev
      where cev.VOT_CODVOT = P_CODVOT;
  END;

  -- Obtenci de l'estat actual d'una votaci.
  --
  -- %param P_CODVOT Codi de la votaci
  --
  -- %return Estat de la votaci. Si la votaci no existeix, retorna el valor null.
  --	         {*} EDI  En edici
  --	         {*} PDT  Pendent dobertura
  --	         {*} INI  Iniciada
  --	         {*} FIN  Finalitzada
  FUNCTION OBTENIR_ESTAT_VOTACIO (P_CODVOT    VOTACIO.CODVOT%TYPE
                                 ) RETURN VOTACIO.ESTVOT%TYPE IS

    Cursor cVot is
      Select vot.ESTVOT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT;

    vEstVot    VOTACIO.ESTVOT%TYPE;
  BEGIN
    Open cVot;
    Fetch cVot into vEstVot;
    Close cVot;

    return vEstVot;
  END;

  -- Comprova si el votaci indicada existeix.
  --
  -- %param P_CODVOT Votaci a comprovar
  --
  -- %return Indicador de si la votaci existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION VOTACIO_EXISTEIX (P_CODVOT    VOTACIO.CODVOT%TYPE
                            ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from VOTACIO vot
                      where vot.CODVOT = P_CODVOT);

    return (vExisteix > 0);
  END;

  -- Dona dalta una votaci.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi
  -- 101           Sha dinformar el ttol
  -- 102           Sha dinformar la descripci
  -- 103           Sha dinformar la data de publicaci
  -- 104           Sha dinformar la data dinici de votaci
  -- 105           Sha dinformar la data de fi de votaci
  -- 106           Sha dinformar el nom del president de la mesa virtual
  -- 107           Ja existeix una votaci amb el mateix codi
  -- 108           La data de publicaci ha de ser posterior a la data de sistema
  -- 109	         La data de publicaci ha de ser com a mnim un setmana abans de la data dinici de la votaci
  -- 110	         La data dinici de votaci ha de ser inferior a la data de fi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_INFTITVOT	   Ttol de la votaci
  -- %param P_DESVOT	     Descripci de la votaci
  -- %param P_DATPUBVOT	   Data de publicaci.
  -- %param P_DATINIVOT	   Data dinici de la votaci.
  -- %param P_DATFINVOT	   Data de fi de la votaci
  -- %param P_INFNOMPRS	   Nom del president de la mesa virtual
  -- %param P_INFWEB	     URL de la web on hi ha instruccions detallades
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_ALTA (P_CODVOT           VOTACIO.CODVOT%TYPE,
                          P_INFTITVOT        VOTACIO.INFTITVOT%TYPE,
                          P_DESVOT           VOTACIO.DESVOT%TYPE,
                          P_DATPUBVOT        VOTACIO.DATPUBVOT%TYPE,
                          P_DATINIVOT        VOTACIO.DATINIVOT%TYPE,
                          P_DATFINVOT        VOTACIO.DATFINVOT%TYPE,
                          P_INFNOMPRS        VOTACIO.INFNOMPRS%TYPE,
                          P_INFWEB           VOTACIO.INFWEB%TYPE,
                          P_RSP          OUT VARCHAR2
                         ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rVot          VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIO_ALTA', 'Alta d''una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFTITVOT     : ' || P_INFTITVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DESVOT        : ' || P_DESVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATPUBVOT     : ' || P_DATPUBVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATINIVOT     : ' || P_DATINIVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATFINVOT     : ' || P_DATFINVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFNOMPRS     : ' || P_INFNOMPRS);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFWEB        : ' || P_INFWEB);

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_INFTITVOT is null then
      P_RSP := '101-El TITOL de la votaci s obligatori.';
    elsif P_DESVOT is null then
      P_RSP := '102-La DESCRIPCIO de la votaci s obligatoria.';
    elsif P_DATPUBVOT is null then
      P_RSP := '103-La DATA PUBLICACI de la votaci s obligatoria.';
    elsif P_DATINIVOT is null then
      P_RSP := '104-La DATA INICI de la votaci s obligatoria.';
    elsif P_DATFINVOT is null then
      P_RSP := '105-La DATA FI de la votaci s obligatoria.';
    elsif P_INFNOMPRS is null then
      P_RSP := '106-El NOM PRESIDENT de la mesa virtual s obligatori.';
    elsif VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '107-Ja existeix una votaci amb el CODI ' || P_CODVOT;
    elsif P_DATPUBVOT <= sysdate then
      P_RSP := '108-La DATA PUBLICACI ha de ser posterior a la data de sistema.';
    elsif P_DATPUBVOT > (P_DATINIVOT - 7) then
      P_RSP := '109-La DATA PUBLICACI ha de ser com a mnim una setmana abans de la data d''inici de la votaci.';
    elsif P_DATINIVOT > P_DATFINVOT then
      P_RSP := '110-La DATA INICI de la votaci ha de ser inferior a la DATA FI de la votaci.';
    end if;


    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rVot.CODVOT    := P_CODVOT;
      rVot.INFTITVOT := P_INFTITVOT;
      rVot.DESVOT    := P_DESVOT;
      rVot.ESTVOT    := cEN_EDICIO;
      rVot.DATPUBVOT := P_DATPUBVOT;
      rVot.DATINIVOT := P_DATINIVOT;
      rVot.DATFINVOT := P_DATFINVOT;
      rVot.INFNOMPRS := P_INFNOMPRS;
      rVot.INFWEB    := P_INFWEB;
      rVot.NUMTOTCTD := 0;
      rVot.NUMTOTVOT := 0;
      rVot.PRCPRT    := 0;

      Insert into VOTACIO
        values rVot;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades duna votaci. Noms es permet modificar-ne les dades si
  -- la votaci est en estat en edici. Si no sinforma alguna de les dades,
  -- preval el valor actual daquesta.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         La data de publicaci ha de ser posterior a la data de sistema
  -- 103	         La data de publicaci ha de ser com a mnim un setmana abans de la data dinici de la votaci
  -- 104	         La data dinici de votaci ha de ser inferior a la data de fi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_INFTITVOT	   Ttol de la votaci. Opcional.
  -- %param P_DESVOT	     Descripci de la votaci. Opcional.
  -- %param P_DATPUBVOT	   Data de publicaci. Opcional.
  -- %param P_DATINIVOT	   Data dinici de la votaci. Opcional.
  -- %param P_DATFINVOT	   Data de fi de la votaci. Opcional.
  -- %param P_INFNOMPRS	   Nom del president de la mesa virtual. Opcional.
  -- %param P_INFWEB	     URL de la web on hi ha instruccions detallades. Opcional.
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_MODIFICACIO (P_CODVOT           VOTACIO.CODVOT%TYPE,
                                 P_INFTITVOT        VOTACIO.INFTITVOT%TYPE := null,
                                 P_DESVOT           VOTACIO.DESVOT%TYPE    := null,
                                 P_DATPUBVOT        VOTACIO.DATPUBVOT%TYPE := null,
                                 P_DATINIVOT        VOTACIO.DATINIVOT%TYPE := null,
                                 P_DATFINVOT        VOTACIO.DATFINVOT%TYPE := null,
                                 P_INFNOMPRS        VOTACIO.INFNOMPRS%TYPE := null,
                                 P_INFWEB           VOTACIO.INFWEB%TYPE    := null,
                                 P_RSP          OUT VARCHAR2
                                ) IS

    Cursor cVot is
      Select vot.*
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT
          for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rVot          VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIO_MODIFICACIO', 'Modificaci de dades d''una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFTITVOT     : ' || P_INFTITVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DESVOT        : ' || P_DESVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATPUBVOT     : ' || P_DATPUBVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATINIVOT     : ' || P_DATINIVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DATFINVOT     : ' || P_DATFINVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFNOMPRS     : ' || P_INFNOMPRS);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFWEB        : ' || P_INFWEB);

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Obtenim les dades actuals de la votaci i bloquejem el registre
    Open cVot;
    Fetch cVot into rVot;
    Close cVot;

    -- Assigno els nous valors
    rVot.INFTITVOT := nvl (P_INFTITVOT, rVot.INFTITVOT);
    rVot.DESVOT    := nvl (P_DESVOT,    rVot.DESVOT);
    rVot.DATPUBVOT := nvl (P_DATPUBVOT, rVot.DATPUBVOT);
    rVot.DATINIVOT := nvl (P_DATINIVOT, rVot.DATINIVOT);
    rVot.DATFINVOT := nvl (P_DATFINVOT, rVot.DATFINVOT);
    rVot.INFNOMPRS := nvl (P_INFNOMPRS, rVot.INFNOMPRS);
    rVot.INFWEB    := nvl (P_INFWEB,    rVot.INFWEB);

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '101-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif rVot.DATPUBVOT <= sysdate then
      P_RSP := '102-La DATA PUBLICACI ha de ser posterior a la data de sistema.';
    elsif rVot.DATPUBVOT > (rVot.DATINIVOT - 7) then
      P_RSP := '103-La DATA PUBLICACI ha de ser com a mnim una setmana abans de la data d''inici de la votaci.';
    elsif rVot.DATINIVOT > rVot.DATFINVOT then
      P_RSP := '104-La DATA INICI de la votaci ha de ser inferior a la DATA FI de la votaci.';
    end if;


    -- s tot correcte, realitzem l'actualitzaci a la taula
    If P_RSP = 'OK' then
      Update VOTACIO vot
        set row = rVot
        where vot.CODVOT = P_CODVOT;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa una votaci. Noms es permet donar de baixa votacions que estan
  -- en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         La votaci no es permet esborrar perqu no est en estat en edici
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_BAIXA (P_CODVOT        VOTACIO.CODVOT%TYPE,
                           P_RSP       OUT VARCHAR2
                          ) IS

    Cursor cVot is
      Select vot.ESTVOT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT
        for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    vEstVot       VOTACIO.ESTVOT%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIO_BAIXA', 'Dona de baixa una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Obtenim l'estat actual de la votaci i bloquejem el registre
    Open cVot;
    Fetch cVot into vEstVot;
    Close cVot;

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '101-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif vEstVot != cEN_EDICIO then
      P_RSP := '102-La votaci no es permet esborrar perqu no est en estat "en edici".';
    end if;

    -- s tot correcte, realitzem la baixa.
    If P_RSP = 'OK' then

      -- Esborrem opcions i censos
      ESBORRAR_OPCIONS_VOTACIO (P_CODVOT);
      ESBORRAR_CENSOS_VOTACIO (P_CODVOT);

      -- Esborrem la votaci
      Delete from VOTACIO vot
        where vot.CODVOT = P_CODVOT;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica lestat duna votaci.
  -- Es valida els segent per a cada estat:
  --  	En edici: la data de publicaci ha de ser inferior a la data de sistema
  --  	Pendent obertura: la data de publicaci ha de ser igual o superior a la data de sistema
  --  	Oberta: la data de sistema ha de ser igual o superior a la data dinici de votaci
  --  	Finalitzada: la data de sistema ha de ser superior a la data de fi de votaci
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         Lestat informat no existeix
  -- 103	         Estat incoherent
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_ESTVOT	     Nou estat de la votaci.
  --	                     {*} EDI  En edici
  --	                     {*} PDT  Pendent dobertura
  --	                     {*} INI  Iniciada
  --	                     {*} FIN  Finalitzada
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_CANVIAR_ESTAT (P_CODVOT        VOTACIO.CODVOT%TYPE,
                                   P_ESTVOT        VOTACIO.ESTVOT%TYPE,
                                   P_RSP       OUT VARCHAR2
                                  ) IS

    Cursor cVot is
      Select vot.*
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT
          for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rVot          VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIO_CANVIAR_ESTAT', 'Canvia l''estat d''una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_ESTVOT        : ' || P_ESTVOT);

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Obtenim les dades actuals de la votaci i bloquejem el registre
    Open cVot;
    Fetch cVot into rVot;
    Close cVot;

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '101-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif P_ESTVOT not in ('EDI', 'PDT', 'INI', 'FIN') then
      P_RSP := '102-L''ESTAT ' || P_ESTVOT || ' no existeix.';

    elsif (P_ESTVOT = cEN_EDICIO   and trunc (sysdate) >= rVot.DATPUBVOT) or
          (P_ESTVOT = cPENDENT     and trunc (sysdate) >  rVot.DATPUBVOT) or
          (P_ESTVOT = cINICIADA    and trunc (sysdate) <  rVot.DATINIVOT) or
          (P_ESTVOT = cFINALITZADA and trunc (sysdate) <= rVot.DATFINVOT) then
      P_RSP := '103-Estat incoherent.';
    end if;

    -- s tot correcte, realitzem l'actualitzaci a la taula
    If P_RSP = 'OK' then
      Update VOTACIO vot
        set vot.ESTVOT = P_ESTVOT
        where vot.CODVOT = P_CODVOT;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Incrementa en 1 el nmero de vots emesos en la votaci.
  --
  -- %param P_CODVOT Codi de la votaci a la que es vol incrementar el nombre
  --                 de vots emesos
  PROCEDURE VOTACIO_INCREMENTA_VOT_EMES (P_CODVOT    VOTACIO.CODVOT%TYPE) IS

    Cursor cVot is
      Select vot.NUMTOTVOT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT
          for update nowait;

    vNumTotVot    VOTACIO.NUMTOTVOT%TYPE;
  BEGIN
    Open cVot;
    Fetch cVot into vNumTotVot;

    vNumTotVot := vNumTotVot + 1;

    Update VOTACIO vot
      set vot.NUMTOTVOT = vNumTotVot
      where current of cVot;

    Close cVot;
  END;

  -- Comprova si el votaci indicada existeix.
  --
  -- %param P_CODVOT Votaci a comprovar
  -- %param P_CODOPV Opci votaci a comprovar
  --
  -- %return Indicador de si l'opci existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION OPCIO_EXISTEIX (P_CODVOT    OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                           P_CODOPV    OPCIO_VOTACIO.CODOPV%TYPE
                          ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from OPCIO_VOTACIO opv
                      where opv.VOT_CODVOT = P_CODVOT and
                            opv.CODOPV = P_CODOPV);

    return (vExisteix > 0);
  END;

  -- Dona dalta un opci de la votaci. Noms es permet afegir opcions a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         Sha dinformar el ttol de la opci
  -- 103	         Sha dinformar la descripci de la opci
  -- 104	         La votaci no existeix
  -- 105	         La votaci est en un estat que no permet afegir opcions.
  -- 106	         Ja existeix una opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_INFTITOPV	   Ttol de la opci
  -- %param P_DESOPV	     Descripci de la opci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_ALTA (P_CODVOT           OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                        P_CODOPV           OPCIO_VOTACIO.CODOPV%TYPE,
                        P_INFTITOPV        OPCIO_VOTACIO.INFTITOPV%TYPE,
                        P_DESOPV           OPCIO_VOTACIO.DESOPV%TYPE,
                        P_RSP          OUT VARCHAR2
                       ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rOpv          OPCIO_VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('OPCIO_ALTA', 'Dona d''alta una opci votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODOPV        : ' || P_CODOPV);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFTITOPV     : ' || P_INFTITOPV);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DESOPV        : ' || P_DESOPV);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_CODOPV is null then
      P_RSP := '101-El CODI OPCIO s obligatori.';
    elsif P_INFTITOPV is null then
      P_RSP := '102-El TITOL OPCI s obligatori.';
    elsif P_DESOPV is null then
      P_RSP := '103-La DESCRIPCI OPCI s obligatoria.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '104-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif OBTENIR_ESTAT_VOTACIO (P_CODVOT) != cEN_EDICIO then
      P_RSP := '105-La votaci est en un estat que no permet afegir opcions.';
    elsif OPCIO_EXISTEIX (P_CODVOT, P_CODOPV) then
      P_RSP := '106-Ja existeix una opci amb el CODI ' || P_CODOPV;
    end if;

    -- s tot correcte, realitzem l'aalta a la taula
    If P_RSP = 'OK' then

      rOpv.VOT_CODVOT := P_CODVOT;
      rOpv.CODOPV     := P_CODOPV;
      rOpv.INFTITOPV  := P_INFTITOPV;
      rOpv.DESOPV     := P_DESOPV;
      rOpv.NUMVOT     := 0;
      rOpv.PRCVOT     := 0;

      Insert into OPCIO_VOTACIO opv
        values rOpv;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades dun opci de la votaci. Noms es permet modificar
  -- opcions a les votacions en estat en edici. Per els valors que no
  -- sinformen, es mant el valor actual.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         No existeix cap opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_INFTITOPV	   Ttol de la opci. Opcional.
  -- %param P_DESOPV	     Descripci de la opci. Opcional.
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_MODIFICACIO (P_CODVOT           OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                               P_CODOPV           OPCIO_VOTACIO.CODOPV%TYPE,
                               P_INFTITOPV        OPCIO_VOTACIO.INFTITOPV%TYPE,
                               P_DESOPV           OPCIO_VOTACIO.DESOPV%TYPE,
                               P_RSP          OUT VARCHAR2
                              ) IS

    Cursor cOpv is
      Select opv.*
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT and
              opv.CODOPV = P_CODOPV
          for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rOpv          OPCIO_VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('OPCIO_MODIFICACIO', 'Modifica dades d''una opci votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODOPV        : ' || P_CODOPV);

    If P_INFTITOPV is not null then
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_INFTITOPV     : ' || P_INFTITOPV);
    end if;

    If P_DESOPV is not null then
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_DESOPV        : ' || P_DESOPV);
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Llegim dades de la opci inidcada
    Open cOpv;
    Fetch cOpv into rOpv;
    Close cOpv;

    -- Assigno els nous valors
    rOpv.INFTITOPV := nvl (P_INFTITOPV, rOpv.INFTITOPV);
    rOpv.DESOPV    := nvl (P_DESOPV, rOpv.DESOPV);

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_CODOPV is null then
      P_RSP := '101-El CODI OPCIO s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '102-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif OBTENIR_ESTAT_VOTACIO (P_CODVOT) != cEN_EDICIO then
      P_RSP := '103-La votaci est en un estat que no permet afegir opcions.';
    elsif not OPCIO_EXISTEIX (P_CODVOT, P_CODOPV) then
      P_RSP := '104-No existeix cap opci amb el CODI ' || P_CODOPV;
    end if;

    -- s tot correcte, realitzem l'actualitzaci a la taula
    If P_RSP = 'OK' then

      Update OPCIO_VOTACIO opv
        set row = rOpv
        where opv.VOT_CODVOT = P_CODVOT and
              opv.CODOPV = P_CODOPV;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa un opci de la votaci. Noms es permet donar de baixa opcions
  -- a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         No existeix cap opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_BAIXA (P_CODVOT        OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                         P_CODOPV        OPCIO_VOTACIO.CODOPV%TYPE,
                         P_RSP       OUT VARCHAR2
                        ) IS

    Cursor cOpv is
      Select 1
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT and
              opv.CODOPV = P_CODOPV
          for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('OPCIO_BAIXA', 'Dona de baixa una opci votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODOPV        : ' || P_CODOPV);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Bloquejem la opci a eliminar
    Open cOpv;
    Close cOpv;

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_CODOPV is null then
      P_RSP := '101-El CODI OPCIO s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '102-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif OBTENIR_ESTAT_VOTACIO (P_CODVOT) != cEN_EDICIO then
      P_RSP := '103-La votaci est en un estat que no permet afegir opcions.';
    elsif not OPCIO_EXISTEIX (P_CODVOT, P_CODOPV) then
      P_RSP := '104-No existeix cap opci amb el CODI ' || P_CODOPV;
    end if;

    -- s tot correcte, realitzem l'actualitzaci a la taula
    If P_RSP = 'OK' then

      Delete from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT and
              opv.CODOPV = P_CODOPV;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Incrementa en 1 el nmero de vots emesos a la opci indicada.
  --
  -- %param P_CODVOT Codi de la votaci
  -- %param P_CODOPV Codi de la que es vol incrementar el nombre de vots rebuts
  PROCEDURE OPCIO_INCREMENTA_VOT_EMES (P_CODVOT    VOTACIO.CODVOT%TYPE,
                                       P_CODOPV    OPCIO_VOTACIO.CODOPV%TYPE
                                      ) IS

    Cursor cOpv is
      Select opv.NUMVOT
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT and
              opv.CODOPV = P_CODOPV
          for update nowait;

    vNumVot    OPCIO_VOTACIO.NUMVOT%TYPE;
  BEGIN
    Open cOpv;
    Fetch cOpv into vNumVot;

    vNumVot := vNumVot + 1;

    Update OPCIO_VOTACIO opv
      set opv.NUMVOT = vNumVot
      where current of cOpv;

    Close cOpv;
  END;

  -- Comprova si el cens indicat est assignat a la votaci
  --
  -- %param P_CODVOT Votaci a comprovar
  -- %param P_CODCEN Cens a comprovar
  --
  -- %return Indicador de si el cens indicat est assignat a la votaci o no.
  --          {*} TRUE   Associat a la votaci
  --          {*} FALSE  No associat a la votaci
  FUNCTION CENS_VOTACIO_EXISTEIX (P_CODVOT    CENS_VOTACIO.VOT_CODVOT%TYPE,
                                  P_CODCEN    CENS_VOTACIO.CEN_CODCEN%TYPE
                                 ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from CENS_VOTACIO cev
                      where cev.VOT_CODVOT = P_CODVOT and
                            cev.CEN_CODCEN = P_CODCEN);

    return (vExisteix > 0);
  END;

  -- Comprova si la votaci t assignat algun cens que englovi
  -- el cens indicat (segons la jerarquia de censos).
  --
  -- %param P_CODVOT Votaci a comprovar
  -- %param P_CODCEN Cens a comprovar
  --
  -- %return Indicador de si la votaci t assignat algun cens superior a l'indicat o no
  --          {*} TRUE   T cens superior associat
  --          {*} FALSE  No t cens superior associat
  FUNCTION EXISTIX_CENS_SUPERIOR_VOTACIO (P_CODVOT    CENS_VOTACIO.VOT_CODVOT%TYPE,
                                          P_CODCEN    CENS_VOTACIO.CEN_CODCEN%TYPE
                                         ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from CENS_VOTACIO cev
                      where cev.VOT_CODVOT = P_CODVOT and
                            cev.CEN_CODCEN in (Select cen.CODCEN
                                                 from CENS cen
                                                 where level > 1
                                                 connect by prior cen.CEN_CODCEN = cen.CODCEN
                                                 start with cen.CODCEN = P_CODCEN));

    return (vExisteix > 0);
  END;

  -- Dona dalta un cens a la votaci. Noms es permet donar dalta
  -- censos a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del cens
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         El cens no existeix
  -- 105	         El cens ja est assignat a la votaci
  -- 106	         Ja sha incls un cens de nivell superior a la votaci.
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODCEN	     Codi de cens
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_VOTACIO_ALTA (P_CODVOT       CENS_VOTACIO.VOT_CODVOT%TYPE,
                               P_CODCEN       CENS_VOTACIO.CEN_CODCEN%TYPE,
                               P_RSP       OUT VARCHAR2
                              ) IS

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    rCev          CENS_VOTACIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CENS_VOTACIO_ALTA', 'Dona d''alta un cens en una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCEN        : ' || P_CODCEN);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_CODCEN is null then
      P_RSP := '101-El CODI CENS s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '102-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif OBTENIR_ESTAT_VOTACIO (P_CODVOT) != cEN_EDICIO then
      P_RSP := '103-La votaci est en un estat que no permet afegir censos.';
    elsif not CENS_PKG.CENS_EXISTEIX (P_CODCEN) then
      P_RSP := '104-El CENS no exiteix.';
    elsif CENS_VOTACIO_EXISTEIX (P_CODVOT, P_CODCEN) then
      P_RSP := '105-El cens ja est assignat a la votaci.';
    elsif EXISTIX_CENS_SUPERIOR_VOTACIO (P_CODVOT, P_CODCEN) then
      P_RSP := '106-La votaci ja t assignat un cens de nivell superior a l''indicat.';
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rCev.VOT_CODVOT := P_CODVOT;
      rCev.CEN_CODCEN := P_CODCEN;

      Insert into CENS_VOTACIO ocev
        values rCev;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa un cens a la votaci. Noms es permet donar de baixa censos
  -- a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del cens
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         El cens no existeix
  -- 105	         El cens no est assignat a la votaci
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODCEN	     Codi de cens
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_VOTACIO_BAIXA (P_CODVOT       CENS_VOTACIO.VOT_CODVOT%TYPE,
                                P_CODCEN       CENS_VOTACIO.CEN_CODCEN%TYPE,
                                P_RSP       OUT VARCHAR2
                               ) IS

    Cursor cCev is
      Select 1
        from CENS_VOTACIO cev
        where cev.VOT_CODVOT = P_CODVOT and
              cev.CEN_CODCEN = P_CODCEN;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CENS_VOTACIO_BAIXA', 'Dona de baixa un cens en una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCEN        : ' || P_CODCEN);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-El CODI de la votaci s obligatori.';
    elsif P_CODCEN is null then
      P_RSP := '101-El CODI CENS s obligatori.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '102-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif OBTENIR_ESTAT_VOTACIO (P_CODVOT) != cEN_EDICIO then
      P_RSP := '103-La votaci est en un estat que no permet afegir censos.';
    elsif not CENS_PKG.CENS_EXISTEIX (P_CODCEN) then
      P_RSP := '104-El CENS no exiteix.';
    elsif not CENS_VOTACIO_EXISTEIX (P_CODVOT, P_CODCEN) then
      P_RSP := '105-El cens no est assignat a la votaci.';
    end if;

    -- s tot correcte, esborrem el registre
    If P_RSP = 'OK' then

      -- Obtenim bloqueig
      Open cCev;
      Close cCev;

      -- Esborrem
      Delete from CENS_VOTACIO cev
        where cev.VOT_CODVOT = P_CODVOT and
              cev.CEN_CODCEN = P_CODCEN;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Comprova si un ciutad t dret a votar en una votaci determinada.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del ciutad
  -- 102	         La votaci no existeix
  -- 103	         El ciutad no existeix
  -- 104           El ciutad no s major d'edat
  -- 900	         Error inesperat
  --
  -- %param P_CODCTD	          Codi del ciutad
  -- %param P_CODVOT	          Codi de votaci
  -- %param P_DRET_A_VOT   OUT  Indica si el ciutad t dret a votar en la votaci
  --                              {*} TRUE  T dret a vot
  --                              {*} FALSE No t dret a vot
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_AMB_DRET_A_VOT (P_CODCTD            CIUTADA.CODCTD%TYPE,
                                    P_CODVOT            VOTACIO.CODVOT%TYPE,
                                    P_DRET_A_VOT    OUT BOOLEAN,
                                    P_RSP           OUT VARCHAR2
                                   ) IS

    Cursor cCtd is
      Select ctd.DATNAC, ctd.EST_CODEST, ctd.REG_CODREG, ctd.LOC_CODLOC
        from CIUTADA ctd
        where ctd.CODCTD = P_CODCTD;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    vDatNac       CIUTADA.DATNAC%TYPE;
    vCodEst       ESTAT.CODEST%TYPE;
    vCodReg       REGIO.CODREG%TYPE;
    vCodLoc       LOCALITAT.CODLOC%TYPE;
    vCodCenGlb    CENS.CODCEN%TYPE;
    vCodCenEst    CENS.CODCEN%TYPE;
    vCodCenReg    CENS.CODCEN%TYPE;
    vCodCenLoc    CENS.CODCEN%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CIUTADA_AMB_DRET_A_VOT', 'Comprova si un ciutad pot votar');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP 	     := 'OK';
    P_DRET_A_VOT := false;

    -- Obtenim dades del ciutad
    Open cCtd;
    Fetch cCtd into vDatNac, vCodEst, vCodReg, vCodLoc;
    Close cCtd;

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CIUTADA s obligatori.';
    elsif P_CODVOT is null then
      P_RSP := '101-La VOTACI s obligatoria.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '102-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif not CIUTADA_PKG.CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '103-No exiteix cap ciutad amb el CODI ' || P_CODCTD;
    elsif ((trunc (sysdate) - trunc (vDatNac)) / 365) < 18 then
      P_RSP := '104-El ciutad no s major d''edat.';
    end if;

    -- s tot correcte, comprovem si existeix el ciutad
    If P_RSP = 'OK' then

      -- Mirem si algun dels censos global, estatal, regional o local
      -- estan associats a la votaci
      vCodCenGlb := CENS_PKG.OBTENIR_CENS_GLOBAL;
      vCodCenEst := CENS_PKG.OBTENIR_CENS_ESTATAL (vCodEst);
      vCodCenReg := CENS_PKG.OBTENIR_CENS_REGIONAL (vCodEst, vCodReg);
      vCodCenLoc := CENS_PKG.OBTENIR_CENS_LOCAL (vCodEst, vCodReg, vCodLoc);

      If CENS_VOTACIO_EXISTEIX (P_CODVOT, vCodCenGlb) or
         CENS_VOTACIO_EXISTEIX (P_CODVOT, vCodCenEst) or
         CENS_VOTACIO_EXISTEIX (P_CODVOT, vCodCenReg) or
         CENS_VOTACIO_EXISTEIX (P_CODVOT, vCodCenLoc) then
        P_DRET_A_VOT := true;
      end if;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Versi del procediment anterior per a poder usar-lo en sentncies SQL
  --
  -- %param P_CODCTD Codi de ciutad
  -- %param P_CODVOT Codi de votaci
  --
  -- %return Indicador de si t dret a votar o no.
  --           {*} 1 T dret a votar
  --           {*} 2 No t dret a votar
  FUNCTION CIUTADA_AMB_DRET_A_VOT (P_CODCTD            CIUTADA.CODCTD%TYPE,
                                   P_CODVOT            VOTACIO.CODVOT%TYPE
                                  ) RETURN NUMBER IS
    vDretAVot    boolean;
    vRsp         varchar2(4000);
    vRes         number;
  BEGIN
    CIUTADA_AMB_DRET_A_VOT (P_CODCTD     => P_CODCTD,
                            P_CODVOT     => P_CODVOT,
                            P_DRET_A_VOT => vDretAVot,
                            P_RSP        => vRsp);
                            
    If vRsp = 'OK' then
      If vDretAVot then
        vRes := 1;
      else
        vRes := 0;
      end if;
    else
      raise_application_error ('-20900', 'Error inesperat');
    end if;
    
    return vRes;
  END;

  -- Comprova si el ciutad indicat ja ha ems el seu vot a en la votaci
  --
  -- %param P_CODCTD Ciutad
  -- %param P_CODVOT Votaci a comprovar
  --
  -- %return Indicador de si el ciutad ha ems el seu vot o no
  --          {*} TRUE   Ha ems el vot
  --          {*} FALSE  No ha ems el vot
  FUNCTION VOT_EMES_EXISTEIX (P_CODCTD    CIUTADA.CODCTD%TYPE,
                              P_CODVOT    OPCIO_VOTACIO.VOT_CODVOT%TYPE
                             ) RETURN BOOLEAN IS
    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from VOT_EMES voe
                      where voe.CTD_CODCTD = P_CODCTD and
                            voe.VOT_CODVOT = P_CODVOT);

    return (vExisteix > 0);
  END;

  -- Comprova si un ciutad t dret a votar en una votaci determinada.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi del ciutad
  -- 101	         Sha dinformar el codi de la votaci
  -- 102	         Sha dinformar el codi de la opci de votaci
  -- 103	         El ciutad no existeix
  -- 104	         La votaci no existeix
  -- 105	         La opci de votaci no existeix
  -- 106	         El ciutad no t dret a votar
  -- 107	         El perode de votaci ha finalitzat
  -- 108	         El ciutad ja ha ems el seu vot
  -- 900	         Error inesperat
  --
  -- %param P_CODCTD	          Codi de la votaci
  -- %param P_CODVOT	          Codi del cens
  -- %param P_CODOPV	          Codi opci votaci escollida
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_EMET_VOT (P_CODCTD        CIUTADA.CODCTD%TYPE,
                              P_CODVOT        VOTACIO.CODVOT%TYPE,
                              P_CODOPV        OPCIO_VOTACIO.CODOPV%TYPE,
                              P_RSP       OUT VARCHAR2
                             ) IS

    Cursor cVot is
      Select vot.DATFINVOT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT;

    vIdeLog          LOG_INFO.IDELOG%TYPE;
    vTeDretAVotar    boolean;
    vDatFinVot       VOTACIO.DATFINVOT%TYPE;
    rVoe             VOT_EMES%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('CIUTADA_EMET_VOT', 'Executa processos votaci ciutad');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODCTD        : ' || P_CODCTD);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODOPV        : ' || P_CODOPV);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP 	     := 'OK';

    -- Llegir dades de la votaci
    Open cVot;
    Fetch cVot into vDatFinVot;
    Close cVot;

    -- Comprovacions prvies
    If P_CODCTD is null then
      P_RSP := '100-El CIUTADA s obligatori.';
    elsif P_CODVOT is null then
      P_RSP := '101-La VOTACI s obligatoria.';
    elsif P_CODOPV is null then
      P_RSP := '102-La OPCI VOTACI s obligatoria.';
    elsif not CIUTADA_PKG.CIUTADA_EXISTEIX (P_CODCTD) then
      P_RSP := '103-No exiteix cap ciutad amb el CODI ' || P_CODCTD;
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '104-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif not OPCIO_EXISTEIX (P_CODVOT, P_CODOPV) then
      P_RSP := '105-No existeix cap opci de la votaci amb el CODI ' || P_CODOPV;
    elsif trunc (vDatFinVot) < trunc (sysdate) then
      P_RSP := '107-El perode de votaci ha finalitzat.';
    elsif VOT_EMES_EXISTEIX (P_CODCTD, P_CODVOT) then
      P_RSP := '108-El ciutad ja ha ems el seu vot.';
    else
      CIUTADA_AMB_DRET_A_VOT (P_CODCTD, P_CODVOT, vTeDretAVotar, P_RSP);

      If P_RSP = 'OK' and not vTeDretAVotar then
         P_RSP := '106-El ciutad no t dret a votar en aquesta votaci';
      end if;
    end if;

    -- s tot correcte, comprovem si existeix el ciutad
    If P_RSP = 'OK' then

      -- Donem d'alta el vot
      rVoe.VOT_CODVOT := P_CODVOT;
      rVoe.CTD_CODCTD := P_CODCTD;

      Insert into VOT_EMES
        values rVoe;

      -- Incrementem els comptados de votaci i de la opci
      VOTACIO_INCREMENTA_VOT_EMES (P_CODVOT);
      OPCIO_INCREMENTA_VOT_EMES (P_CODVOT, P_CODOPV);
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Procs de tancament duna votaci. Realitza el cmput de participaci i llana
  -- el procs dextracci de les dades estadstiques. Noms es pot finalitzar votacions
  -- que estan en estat oberta i que la data de sistema es superior a la data de fi de
  -- votaci.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         La votaci no existeix
  -- 102	         La votaci no est en un estat que permeti realitzar el tancament.
  -- 103	         El perode de votaci no ha finalitzat.
  -- 104	         Error en el procs destadstiques.
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	          Codi del cens
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_TANCAR (P_CODVOT        VOTACIO.CODVOT%TYPE,
                            P_RSP       OUT VARCHAR2
                           ) IS

    Cursor cVot is
      Select vot.ESTVOT, vot.DATFINVOT, vot.NUMTOTVOT
        from VOTACIO vot
        where vot.CODVOT = P_CODVOT
          for update nowait;

    vIdeLog       LOG_INFO.IDELOG%TYPE;
    vEstVot       VOTACIO.ESTVOT%TYPE;
    vDatFinVot    VOTACIO.DATFINVOT%TYPE;
    vNumTotVot    VOTACIO.NUMTOTVOT%TYPE;
    vNumTotCtd    VOTACIO.NUMTOTCTD%TYPE;
    vPrcPrt       VOTACIO.PRCPRT%TYPE;
    vPrcVot       OPCIO_VOTACIO.PRCVOT%TYPE;
    vPrcVotTot    OPCIO_VOTACIO.PRCVOT%TYPE;
    vRspEsd       varchar2(4000);
    vMaxPrcVot    OPCIO_VOTACIO.PRCVOT%TYPE;
    vMinPrcVot    OPCIO_VOTACIO.PRCVOT%TYPE;
    vMaxCodOpv    OPCIO_VOTACIO.CODOPV%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('VOTACIO_TANCAR', 'Tanca una votaci');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODVOT        : ' || P_CODVOT);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP 	     := 'OK';

    -- Llegir dades de la votaci
    Open cVot;
    Fetch cVot into vEstVot, vDatFinVot, vNumTotVot;
    Close cVot;

    -- Comprovacions prvies
    If P_CODVOT is null then
      P_RSP := '100-La VOTACI s obligatoria.';
    elsif not VOTACIO_EXISTEIX (P_CODVOT) then
      P_RSP := '101-No existeix cap votaci amb el CODI ' || P_CODVOT;
    elsif vEstVot != cINICIADA then
      P_RSP := '102-La votaci est en un estat que no permet ser tancada.';
    elsif trunc (vDatFinVot) > trunc (sysdate) then
      P_RSP := '103-El perode de votaci no ha finalitzat.';
    end if;

    -- s tot correcte, comprovem si existeix el ciutad
    If P_RSP = 'OK' then

      -- Per a cada cens associat a la votaci, calculem els ciutadans que
      -- hi tenen dret
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Clcul del nombre de ciutadans amb dret a vot');

      vNumTotCtd := 0;

      For rCev in (Select cev.CEN_CODCEN
                     from CENS_VOTACIO cev
                     where cev.VOT_CODVOT = P_CODVOT) loop

        vNumTotCtd := vNumTotCtd + CENS_PKG.NOMBRE_CIUTADANS_CENS (rCev.CEN_CODCEN, vDatFinVot);
      end loop;

      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Total ciutadans amb dret a vot:' || vNumTotCtd);

      -- Calculem el % de participaci:
      --  total de vots emesos sobre el total de ciutadans amb dret a vot
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Clcul del percentatge de participaci');

      vPrcPrt := round ((vNumTotVot / vNumTotCtd) * 100, 2);

      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Participaci: ' || vPrcPrt || '%');

      -- Actualitzem les dades a la votaci
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Actualitzar les dades de la votaci.');

      Update VOTACIO vot
        set vot.ESTVOT = cFINALITZADA,
            vot.NUMTOTCTD = vNumTotCtd,
            vot.PRCPRT = vPrcPrt
        where vot.CODVOT = P_CODVOT;


      -- Per a cada opci de la votaci, calculem el percentatge de vots rebuts
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Calcular percentetatge de vots rebuts per cada opci de la votaci');

      vPrcVotTot := 0;
      vMaxPrcVot := 0;
      vMinPrcVot := 100;
      vMaxCodOpv := null;

      For rOpv in (Select opv.CODOPV, opv.NUMVOT
                     from OPCIO_VOTACIO opv
                     where opv.VOT_CODVOT = P_CODVOT
                       for update nowait) loop

        -- Calculem el % de vots rebuts
        vPrcVot := round ((rOpv.NUMVOT / vNumTotVot) * 100, 2);

        LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');
        LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Opci:       ' || rOpv.CODOPV);
        LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Vots rebuts: ' || rOpv.NUMVOT);
        LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Percentatge: ' || vPrcVot || '%');

        -- Control sobre el percentate. Si passem de 100, l'ajustem
        If (vPrcVotTot + vPrcVot) > 100 then
          vPrcVot := 100 - vPrcVotTot;
          LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'S''ajusta el percentatge de vots rebuts per tal de no sobrepassar el 100%');
        end if;

        vPrcVotTot := vPrcVotTot + vPrcVot;

        -- Actualitzem el valor a la taula
        LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Actualitzar dades de l''opaci de la votaci');

        Update OPCIO_VOTACIO opv
          set opv.PRCVOT = vPrcVot,
              opv.FLGGNY = 'N'
          where opv.VOT_CODVOT = P_CODVOT and
                opv.CODOPV = rOpv.CODOPV;

        -- Controlem difrencia de vots
        If vPrcVot  > vMaxPrcVot then
          vMaxPrcVot := vPrcVot;
          vMaxCodOpv := rOpv.CODOpV;
        end if;
        
        If vPrcVot < vMinPrcVOt then
          vMinPrcVot := vPrcVot;
        end if;
      end loop;

      -- Marquem la opci guanyadora
      Update OPCIO_VOTACIO opv
         set opv.FLGGNY = 'S'
         where opv.VOT_CODVOT = P_CODVOT and
               opv.CODOPV = vMaxCodOpv;

      -- Actualitzem la diferncia entre la opci ms votada i la que menys
      Update VOTACIO vot
        set vot.NUMDIFOPV = (vMaxPrcVot - vMinPrcVot)
        where vot.CODVOT = P_CODVOT;

      -- Executar Extracci dades estadstiques votaci
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Calcular estadstiques');

      ESTADISTIQUES_PKG.EXTREURE_DADES_VOTACIO (P_CODVOT, vRspEsd);

      -- Si s'ha produt errors, els registrem
      If vRspEsd != 'OK' then
        P_RSP := '104' || substr (vRspEsd, 4);
      end if;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Retorna el codi de la opci que t ms percentatge de votaci.
  --
  -- %param P_CODVOT Codi de la votaci
  --
  -- %return Codi de la opci amb ms vots.
  FUNCTION OPCIO_AMB_MES_VOTS (P_CODVOT    VOTACIO.CODVOT%TYPE
                              ) RETURN OPCIO_VOTACIO.CODOPV%TYPE IS

    Cursor cOpv is
      Select opv.CODOPV, opv.NUMVOT
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT;

    vCodOpv       OPCIO_VOTACIO.CODOPV%TYPE;
    vNumVot       OPCIO_VOTACIO.NUMVOT%TYPE;
    vMaxCodOpv    OPCIO_VOTACIO.CODOPV%TYPE;
    vMaxNumVot    OPCIO_VOTACIO.NUMVOT%TYPE;
  BEGIN
    vMaxNumVot := 0;
    
    Open cOpv;
    Fetch cOpv into vCodOpv, vNumVot;
    While cOpv%FOUND loop
    
      If vNumVot > vMaxNumVot then
        vMaxNumVot := vNumVot;
        vMaxCodOpv := vCodOpv;        
      end if;
      
      Fetch cOpv into vCodOpv, vNumVot;
    end loop;
    Close cOpv;
    
    return vMaxCodOpv;
  END;

  -- Retorna el codi de la opci que t menys percentatge de votaci.
  --
  -- %param P_CODVOT Codi de la votaci
  --
  -- %return Codi de la opci amb menys vots.
  FUNCTION OPCIO_AMB_MENYS_VOTS (P_CODVOT    VOTACIO.CODVOT%TYPE
                                ) RETURN OPCIO_VOTACIO.CODOPV%TYPE IS

    Cursor cOpv is
      Select opv.CODOPV, opv.NUMVOT
        from OPCIO_VOTACIO opv
        where opv.VOT_CODVOT = P_CODVOT;

    vCodOpv       OPCIO_VOTACIO.CODOPV%TYPE;
    vNumVot       OPCIO_VOTACIO.NUMVOT%TYPE;
    vMinCodOpv    OPCIO_VOTACIO.CODOPV%TYPE;
    vMinNumVot    OPCIO_VOTACIO.NUMVOT%TYPE;
  BEGIN
    vMinNumVot := 0;
    
    Open cOpv;
    Fetch cOpv into vCodOpv, vNumVot;
    While cOpv%FOUND loop
    
      If vNumVot < vMinNumVot then
        vMinNumVot := vNumVot;
        vMinCodOpv := vCodOpv;        
      end if;
      
      Fetch cOpv into vCodOpv, vNumVot;
    end loop;
    Close cOpv;
    
    return vMinCodOpv;
  END;


  
end;
/

exit
/

