create or replace package VOTACIO_PKG is

  -- Estats d'una votaci
  cEN_EDICIO      constant varchar2(3) := 'EDI';
  cPENDENT        constant varchar2(3) := 'PDT';
  cINICIADA       constant varchar2(3) := 'INI';
  cFINALITZADA    constant varchar2(3) := 'FIN';

  -- Comprova si el votaci indicada existeix.
  --
  -- %param P_CODVOT Votaci a comprovar
  --
  -- %return Indicador de si la votaci existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION VOTACIO_EXISTEIX (P_CODVOT    VOTACIO.CODVOT%TYPE
                            ) RETURN BOOLEAN;

  -- Dona dalta una votaci.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100           Sha dinformar  el codi
  -- 101           Sha dinformar el ttol
  -- 102           Sha dinformar la descripci
  -- 103           Sha dinformar la data de publicaci
  -- 104           Sha dinformar la data dinici de votaci
  -- 105           Sha dinformar la data de fi de votaci
  -- 106           Sha dinformar el nom del president de la mesa virtual
  -- 107           Ja existeix una votaci amb el mateix codi
  -- 108           La data de publicaci ha de ser posterior a la data de sistema
  -- 109           La data de publicaci ha de ser com a mnim un setmana abans de la data dinici de la votaci
  -- 110           La data dinici de votaci ha de ser inferior a la data de fi
  -- 900           Error inesperat
  --
  -- %param P_CODVOT       Codi de la votaci
  -- %param P_INFTITVOT     Ttol de la votaci
  -- %param P_DESVOT       Descripci de la votaci
  -- %param P_DATPUBVOT     Data de publicaci.
  -- %param P_DATINIVOT     Data dinici de la votaci.
  -- %param P_DATFINVOT     Data de fi de la votaci
  -- %param P_INFNOMPRS     Nom del president de la mesa virtual
  -- %param P_INFWEB       URL de la web on hi ha instruccions detallades
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_ALTA (P_CODVOT           VOTACIO.CODVOT%TYPE,
                          P_INFTITVOT        VOTACIO.INFTITVOT%TYPE,
                          P_DESVOT           VOTACIO.DESVOT%TYPE,
                          P_DATPUBVOT        VOTACIO.DATPUBVOT%TYPE,
                          P_DATINIVOT        VOTACIO.DATINIVOT%TYPE,
                          P_DATFINVOT        VOTACIO.DATFINVOT%TYPE,
                          P_INFNOMPRS        VOTACIO.INFNOMPRS%TYPE,
                          P_INFWEB           VOTACIO.INFWEB%TYPE,
                          P_RSP          OUT VARCHAR2);

  -- Modifica les dades duna votaci. Noms es permet modificar-ne les dades si
  -- la votaci est en estat en edici. Si no sinforma alguna de les dades,
  -- preval el valor actual daquesta.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         La data de publicaci ha de ser posterior a la data de sistema
  -- 103	         La data de publicaci ha de ser com a mnim un setmana abans de la data dinici de la votaci
  -- 104	         La data dinici de votaci ha de ser inferior a la data de fi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_INFTITVOT	   Ttol de la votaci. Opcional.
  -- %param P_DESVOT	     Descripci de la votaci. Opcional.
  -- %param P_DATPUBVOT	   Data de publicaci. Opcional.
  -- %param P_DATINIVOT	   Data dinici de la votaci. Opcional.
  -- %param P_DATFINVOT	   Data de fi de la votaci. Opcional.
  -- %param P_INFNOMPRS	   Nom del president de la mesa virtual. Opcional.
  -- %param P_INFWEB	     URL de la web on hi ha instruccions detallades. Opcional.
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_MODIFICACIO (P_CODVOT           VOTACIO.CODVOT%TYPE,
                                 P_INFTITVOT        VOTACIO.INFTITVOT%TYPE := null,
                                 P_DESVOT           VOTACIO.DESVOT%TYPE    := null,
                                 P_DATPUBVOT        VOTACIO.DATPUBVOT%TYPE := null,
                                 P_DATINIVOT        VOTACIO.DATINIVOT%TYPE := null,
                                 P_DATFINVOT        VOTACIO.DATFINVOT%TYPE := null,
                                 P_INFNOMPRS        VOTACIO.INFNOMPRS%TYPE := null,
                                 P_INFWEB           VOTACIO.INFWEB%TYPE    := null,
                                 P_RSP          OUT VARCHAR2);

  -- Dona de baixa una votaci. Noms es permet donar de baixa votacions que estan
  -- en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         La votaci no es permet esborrar perqu no est en estat en edici
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_BAIXA (P_CODVOT        VOTACIO.CODVOT%TYPE,
                           P_RSP       OUT VARCHAR2);

  -- Modifica lestat duna votaci.
  -- Es valida els segent per a cada estat:
  --  	En edici: la data de publicaci ha de ser inferior a la data de sistema
  --  	Pendent obertura: la data de publicaci ha de ser igual o superior a la data de sistema
  --  	Oberta: la data de sistema ha de ser igual o superior a la data dinici de votaci
  --  	Finalitzada: la data de sistema ha de ser superior a la data de fi de votaci
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar  el codi
  -- 101	         No existeix cap votaci amb el codi indicat
  -- 102	         Lestat informat no existeix
  -- 103	         Estat incoherent
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_ESTVOT	     Nou estat de la votaci.
  --	                     {*} EDI  En edici
  --	                     {*} PDT  Pendent dobertura
  --	                     {*} INI  Iniciada
  --	                     {*} FIN  Finalitzada
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_CANVIAR_ESTAT (P_CODVOT        VOTACIO.CODVOT%TYPE,
                                   P_ESTVOT        VOTACIO.ESTVOT%TYPE,
                                   P_RSP       OUT VARCHAR2);

  -- Comprova si el votaci indicada existeix.
  --
  -- %param P_CODVOT Votaci a comprovar
  -- %param P_CODOPV Opci votaci a comprovar
  --
  -- %return Indicador de si l'opci existeix o no.
  --          {*} TRUE   Existeix
  --          {*} FALSE  No existeix
  FUNCTION OPCIO_EXISTEIX (P_CODVOT    OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                           P_CODOPV    OPCIO_VOTACIO.CODOPV%TYPE
                          ) RETURN BOOLEAN;

  -- Dona dalta un opci de la votaci. Noms es permet afegir opcions a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         Sha dinformar el ttol de la opci
  -- 103	         Sha dinformar la descripci de la opci
  -- 104	         La votaci no existeix
  -- 105	         La votaci est en un estat que no permet afegir opcions.
  -- 106	         Ja existeix una opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_INFTITOPV	   Ttol de la opci
  -- %param P_DESOPV	     Descripci de la opci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_ALTA (P_CODVOT           OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                        P_CODOPV           OPCIO_VOTACIO.CODOPV%TYPE,
                        P_INFTITOPV        OPCIO_VOTACIO.INFTITOPV%TYPE,
                        P_DESOPV           OPCIO_VOTACIO.DESOPV%TYPE,
                        P_RSP          OUT VARCHAR2);

  -- Modifica les dades dun opci de la votaci. Noms es permet modificar
  -- opcions a les votacions en estat en edici. Per els valors que no
  -- sinformen, es mant el valor actual.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         No existeix cap opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_INFTITOPV	   Ttol de la opci
  -- %param P_DESOPV	     Descripci de la opci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_MODIFICACIO (P_CODVOT           OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                               P_CODOPV           OPCIO_VOTACIO.CODOPV%TYPE,
                               P_INFTITOPV        OPCIO_VOTACIO.INFTITOPV%TYPE,
                               P_DESOPV           OPCIO_VOTACIO.DESOPV%TYPE,
                               P_RSP          OUT VARCHAR2);

  -- Dona de baixa un opci de la votaci. Noms es permet donar de baixa opcions
  -- a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi de la opci de la votaci
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         No existeix cap opci amb aquest codi
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODOPV	     Codi de la opci de la votaci
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE OPCIO_BAIXA (P_CODVOT        OPCIO_VOTACIO.VOT_CODVOT%TYPE,
                         P_CODOPV        OPCIO_VOTACIO.CODOPV%TYPE,
                         P_RSP       OUT VARCHAR2);

  -- Comprova si el cens indicat est assignat a la votaci
  --
  -- %param P_CODVOT Votaci a comprovar
  -- %param P_CODCEN Cens a comprovar
  --
  -- %return Indicador de si el cens indicat est assignat a la votaci o no.
  --          {*} TRUE   Associat a la votaci
  --          {*} FALSE  No associat a la votaci
  FUNCTION CENS_VOTACIO_EXISTEIX (P_CODVOT    CENS_VOTACIO.VOT_CODVOT%TYPE,
                                  P_CODCEN    CENS_VOTACIO.CEN_CODCEN%TYPE
                                 ) RETURN BOOLEAN;


  -- Dona dalta un cens a la votaci. Noms es permet donar dalta
  -- censos a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del cens
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         El cens no existeix
  -- 105	         El cens ja est assignat a la votaci
  -- 106	         Ja sha incls un cens de nivell superior a la votaci.
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODCEN	     Codi de cens
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_VOTACIO_ALTA (P_CODVOT       CENS_VOTACIO.VOT_CODVOT%TYPE,
                               P_CODCEN       CENS_VOTACIO.CEN_CODCEN%TYPE,
                               P_RSP       OUT VARCHAR2);

  -- Dona de baixa un cens a la votaci. Noms es permet donar de baixa censos
  -- a les votacions en estat en edici.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del cens
  -- 102	         La votaci no existeix
  -- 103	         La votaci est en un estat que no permet afegir opcions.
  -- 104	         El cens no existeix
  -- 105	         El cens no est assignat a la votaci
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	     Codi de la votaci
  -- %param P_CODCEN	     Codi de cens
  -- %param P_RSP     OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                       valor OK, sin retorna un codi derror  seguit de la descripci,
  --                       seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CENS_VOTACIO_BAIXA (P_CODVOT       CENS_VOTACIO.VOT_CODVOT%TYPE,
                                P_CODCEN       CENS_VOTACIO.CEN_CODCEN%TYPE,
                                P_RSP       OUT VARCHAR2);

  -- Comprova si un ciutad t dret a votar en una votaci determinada.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         Sha dinformar el codi del ciutad
  -- 102	         La votaci no existeix
  -- 103	         El ciutad no existeix
  -- 900	         Error inesperat
  --
  -- %param P_CODCTD	          Codi del ciutad
  -- %param P_CODVOT	          Codi de votaci
  -- %param P_DRET_A_VOT   OUT  Indica si el ciutad t dret a votar en la votaci
  --                              {*} TRUE  T dret a vot
  --                              {*} FALSE No t dret a vot
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_AMB_DRET_A_VOT (P_CODCTD            CIUTADA.CODCTD%TYPE,
                                    P_CODVOT            VOTACIO.CODVOT%TYPE,
                                    P_DRET_A_VOT    OUT BOOLEAN,
                                    P_RSP           OUT VARCHAR2);

  -- Versi del procediment anterior per a poder usar-lo en sentncies SQL
  --
  -- %param P_CODCTD Codi de ciutad
  -- %param P_CODVOT Codi de votaci
  --
  -- %return Indicador de si t dret a votar o no.
  --           {*} 1 T dret a votar
  --           {*} 2 No t dret a votar
  FUNCTION CIUTADA_AMB_DRET_A_VOT (P_CODCTD            CIUTADA.CODCTD%TYPE,
                                   P_CODVOT            VOTACIO.CODVOT%TYPE
                                  ) RETURN NUMBER;

  -- Comprova si un ciutad t dret a votar en una votaci determinada.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi del ciutad
  -- 101	         Sha dinformar el codi de la votaci
  -- 102	         Sha dinformar el codi de la opci de votaci
  -- 103	         El ciutad no existeix
  -- 104	         La votaci no existeix
  -- 105	         La opci de votaci no existeix
  -- 106	         El ciutad no t dret a votar
  -- 107	         El perode de votaci ha finalitzat
  -- 108	         El ciutad ja ha ems el seu vot
  -- 900	         Error inesperat
  --
  -- %param P_CODCTD	          Codi de la votaci
  -- %param P_CODVOT	          Codi del cens
  -- %param P_CODOPV	          Codi opci votaci escollida
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE CIUTADA_EMET_VOT (P_CODCTD        CIUTADA.CODCTD%TYPE,
                              P_CODVOT        VOTACIO.CODVOT%TYPE,
                              P_CODOPV        OPCIO_VOTACIO.CODOPV%TYPE,
                              P_RSP       OUT VARCHAR2);

  -- Procs de tancament duna votaci. Realitza el cmput de participaci i llana
  -- el procs dextracci de les dades estadstiques. Noms es pot finalitzar votacions
  -- que estan en estat oberta i que la data de sistema es superior a la data de fi de
  -- votaci.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror  Descripci
  -- ------------  -----------------------------------
  -- 100	         Sha dinformar el codi de la votaci
  -- 101	         La votaci no existeix
  -- 102	         La votaci no est en un estat que permeti realitzar el tancament.
  -- 103	         El perode de votaci no ha finalitzat.
  -- 104	         Error en el procs destadstiques.
  -- 900	         Error inesperat
  --
  -- %param P_CODVOT	          Codi del cens
  -- %param P_RSP          OUT  Codi de resposta. Si el procs dalta acabat amb xit, retorn el
  --                            valor OK, sin retorna un codi derror  seguit de la descripci,
  --                            seguint el format CODI-DESCRIPCIӔ
  PROCEDURE VOTACIO_TANCAR (P_CODVOT        VOTACIO.CODVOT%TYPE,
                            P_RSP       OUT VARCHAR2);

  -- Retorna el codi de la opci que t ms percentatge de votaci.
  --
  -- %param P_CODVOT Codi de la votaci
  --
  -- %return Codi de la opci amb ms vots.
  FUNCTION OPCIO_AMB_MES_VOTS (P_CODVOT    VOTACIO.CODVOT%TYPE
                              ) RETURN OPCIO_VOTACIO.CODOPV%TYPE;

  -- Retorna el codi de la opci que t menys percentatge de votaci.
  --
  -- %param P_CODVOT Codi de la votaci
  --
  -- %return Codi de la opci amb menys vots.
  FUNCTION OPCIO_AMB_MENYS_VOTS (P_CODVOT    VOTACIO.CODVOT%TYPE
                                ) RETURN OPCIO_VOTACIO.CODOPV%TYPE;

end;
/

exit
/

